<?php

/**
 * mo_viaebay__feed
 * 
 * abstract feed class
 * provides basic feed functionality
 * 
 * @version   $Id: mo_viaebay__feed.php 139 2012-12-10 16:51:05Z $
 */

/**
 * abstract feed class
 * provides basic feed functionality
 * 
 * @package   mo_viaebay
 * @abstract 
 */
abstract class mo_viaebay__feed
{

  /**
   * stores relations
   * 
   * @var array
   */
  protected $relations = array();

  /**
   * stores feed data
   * 
   * @var array
   */
  protected $data = null;

  /**
   * constructor calls load function
   * 
   * @param array $ids 
   */
  public function __construct($ids = array(), $options = array())
  {
    $this->load($ids, $options);
  }

  /**
   * returns data if set
   * loads and returns data if not set
   * 
   * @return array 
   */
  public function getData()
  {
    if ($this->data !== null)
    {
      return $this->data;
    }
    $this->data = $this->load();
    return $this->data ? $this->data : array();
  }

  /**
   * abstract load function
   * extended by child classes
   * 
   * @param array $ids
   */
  abstract protected function load($ids = array(), $options = array());

  /**
   * returns record with specified Id if existent
   * 
   * @param string $id
   * @return mixed boolean / array 
   */
  public function getRecord($id)
  {
    if (!isset($this->data[$id]))
    {
      return false;
    }
    return $this->data[$id];
  }

  /**
   * handles empty relations
   * 
   * @param array $record
   * @return array 
   */
  public function handleEmptyRelations($record)
  {
    foreach (array_keys($this->relations) as $relation)
    {
      if (isset($record[$relation]['__deferred']))
      {
        $record[$relation] = array();
      }
    }
    return $record;
  }

  /**
   * simplifies realtions
   * 
   * @param array $record
   * @return array 
   */
  protected function simplifyRelations($record)
  {
    foreach ($this->relations as $relation => $relationData)
    {
      if (empty($record[$relation]))
      {
        continue;
      }

      unset($record[$relation]['__metadata']);
      unset($record[$relation]['__deferred']);

      if (empty($relationData['doSimplify']))
      {
        continue;
      }

      $complexRelation   = $record[$relation];
      $record[$relation] = $this->mo_viaebay__rebuildRelationArray($complexRelation);
    }
    return $record;
  }

  /**
   * if awaited list has only 1 element, VIA does not return it as a collection
   * 
   * @param array $response 
   * @return array $result 
   */
  protected function buildCollection($response)
  {
    if (!isset($response['Id']))
    {
      return $response;
    }
    return array($response);
  }

  /**
   * rebuilds relation array
   * 
   * @param array $complexRelation 
   * @return array
   */
  protected function mo_viaebay__rebuildRelationArray($complexRelation)
  {
    $relation = array();

    foreach ($complexRelation as $data)
    {
      $relation[] = $data['Id'];
    }

    return $relation;
  }

}