<?php

/**
 * mo_viaebay__feed_articles
 * 
 * loads and handles synced article ids from via
 * 
 * @version   $Id: mo_viaebay__feed_articles.php 161 2012-12-17 13:20:04Z $
 */

/**
 * loads and handles synced article ids from via
 * 
 * @package   mo_viaebay
 * @extend mo_viaebay__feed 
 */
class mo_viaebay__feed_articles extends mo_viaebay__feed
{

    const OPTION_LOAD_BY_VIA_ID = 1;
    
  /**
   * stores relations
   * 
   * @var array
   */
  protected $relations = array(
      'ProductImages' => array('doSimplify'    => false),
      'ChildProducts' => array('doSimplify'     => false),
      'ParentProducts' => array('doSimplify'       => false),
      'ProductSpecifics' => array('doSimplify' => false),
      'Catalogs'   => array('doSimplify'        => false),
      'ProductVariations' => array('doSimplify' => false),
  );

  /**
   * load and handle article ids from via
   * 
   * @extend load
   * @param array $ids
   * @return array 
   */
  public function load($ids = array(), $options = array())
  {
    $articles = mo_viaebay__main::getInstance()->getInterface()->getArticles($ids, $options);
    $articles = $this->buildCollection($articles);

    //relations
    foreach ($articles as $article)
    {
      unset($article['__metadata']);
      $article = $this->handleEmptyRelations($article);
      $article = $this->simplifyRelations($article);
      $article = $this->prepareAttributes($article);
      $article = $this->prepareVariationRelations($article);
      $article = $this->prepareAdditionalInformation($article);
      $this->data[$article['Id']] = $article;
    }

    return $this->data;
  }

  /**
   * prepare attrbute data (set Name as key)
   * @param array $articleData
   * @return type 
   */
  protected function prepareAttributes($articleData)
  {
    if (empty($articleData['ProductSpecifics']))
    {
      return $articleData;
    }

    $attributes = array();
    foreach ($articleData['ProductSpecifics'] as $specific)
    {
      $attributes[$specific['Name']] = $specific;
    }

    $articleData['ProductSpecifics'] = $attributes;
    return $articleData;
  }

  /**
   * prepare relation data
   *
   * @param array $articleData
   * @return array 
   */
  protected function prepareVariationRelations($articleData)
  {
    if (empty($articleData['ProductVariations']))
    {
      return $articleData;
    }

    $variations = array();

    foreach ($this->buildCollection($articleData['ProductVariations']) as $variation)
    {
      $variations[$variation['Id']]                              = $variation;
      $variations[$variation['Id']]['ProductVariationSpecifics'] = $this->prepareSpecs($variation);
    }

    $articleData['ProductVariations'] = $variations;

    return $articleData;
  }

  /**
   * prepare product variant specifics
   *
   * @param array $variation
   * @return array 
   */
  protected function prepareSpecs($variation)
  {
    $specs = array();

    foreach ($this->buildCollection($variation['ProductVariationSpecifics']) as $spec)
    {
      $specs[$spec['Name']] = $spec;
    }

    return $specs;
  }
  protected function prepareAdditionalInformation($articleData)
  {
        if (empty($articleData['OptionalProductAttributes']))
    {
      return $articleData;
    }

    $attributes = array();
    foreach ($articleData['OptionalProductAttributes'] as $specific)
    {
      $attributes[$specific['Name']] = $specific;
    }

    $articleData['OptionalProductAttributes'] = $attributes;
    return $articleData;
  }

}