<?php

/**
 * handler for module activation/deactivation event
 */
class mo_viaebay__events
{

    /**
     * on module activation extend oxaction database table with new coloumns
     */
    public static function onActivate()
    {
        $sqlStatements = array();
        $errors = false;

        $status = self::isDatabaseUpdated();
        if ($status == 0) {
            $sqlStatements = self::getSqlStatements();
        }
        if ($status == 1 || $status == 2) {
            $sqlStatements = self::getMultishopUpdateSqlStatements();
        }
        if ($status == 1) {
            $sqlStatements = self::getArtExtendUpdateSqlStatements($sqlStatements);
        }
        if (self::isVersionUnder131()) {
            $sqlStatements = self::get131DBUpdate($sqlStatements);
        }

        foreach ($sqlStatements as $sqlStatement) {
            try {
                oxDb::getDb()->Execute($sqlStatement);
            } catch (oxAdoDbException $exc) {
                oxRegistry::get('oxUtilsView')
                        ->addErrorToDisplay(oxRegistry::getLang()
                                ->translateString($sqlStatement));
                oxRegistry::get('oxUtilsView')
                        ->addErrorToDisplay(oxRegistry::getLang()
                                ->translateString($exc));
                oxRegistry::get('oxUtilsView')
                        ->addErrorToDisplay(oxRegistry::getLang()
                                ->translateString('MO_VIAEBAY__DATABASE_INSTALLATION_ERROR'));
                $errors = true;
            }
        }
        self::checkArticles2ViaIndex();
        self::checkCategory2ViaIndex();
        self::addTrackingSyncColumn();
        self::addInventorySyncDateColumn();

        if (!$errors) {
            $metaDataHandler = oxNew('oxDbMetaDataHandler');
            $metaDataHandler->updateViews();
        }
    }

    /*
     * Checking if the database has...
     * return 0: first time installed viaebay
     * return 1: 1.0.0 version of viaebay
     * return 2: added articleextend  
     * return 3: EE-version of viaebay
     *      * 
     * @return int
     */

    protected static function isDatabaseUpdated()
    {
        $dbName = oxRegistry::get("oxConfigFile")->getVar('dbName');
        $oxarticlesColumns = oxDb::getDb()->Execute("SHOW COLUMNS FROM `oxarticles` FROM `"
                . $dbName . "`  LIKE '%mo_viaebay%'");
        $oxartextendsColumns = oxDb::getDb()->Execute("SHOW COLUMNS FROM `oxartextends` FROM `"
                . $dbName . "`  LIKE '%mo_viaebay%'");
        if (($oxarticlesColumns->RecordCount() >= 3) && ($oxartextendsColumns->RecordCount() == 1)) {
            return 2;
        }
        if (($oxarticlesColumns->RecordCount() >= 3) && ($oxartextendsColumns->RecordCount() == 0)) {
            return 1;
        }
        $oxorderColumns = oxDb::getDb()->Execute("SHOW COLUMNS FROM `oxorder` FROM `"
                . $dbName . "`  LIKE '%mo_viaebay%'");
        if ($oxorderColumns->RecordCount() > 0) {
            return 3;
        }
        return 0;
    }

    protected static function getSqlStatements()
    {
        $sqlStatements['updateOxOrder_viaID'] = "ALTER TABLE `oxorder` ADD `MO_VIAEBAY__VIA_ID` INT NULL ,"
                . "ADD UNIQUE (`MO_VIAEBAY__VIA_ID`);";
        $sqlStatements['updateOxOrder_LAST_SYNC'] = "ALTER TABLE `oxorder` ADD `MO_VIAEBAY__LAST_SYNC` DATETIME NOT NULL;";

        $sqlStatements['updateOxOrderArticles_VIA_ID'] = "ALTER TABLE `oxorderarticles` ADD `MO_VIAEBAY__VIA_ID` INT NULL ,"
                . "ADD UNIQUE (`MO_VIAEBAY__VIA_ID`);";
        $sqlStatements['create_assigned_categories'] = "CREATE TABLE IF NOT EXISTS `mo_viaebay__assigned_categories` ("
                . "`OXID` char(32) CHARACTER SET latin1 COLLATE latin1_general_ci NOT NULL,"
                . "`OXOBJECTID` char(32) CHARACTER SET latin1 COLLATE latin1_general_ci NOT NULL DEFAULT '',"
                . "`OXCATNID` char(32) CHARACTER SET latin1 COLLATE latin1_general_ci NOT NULL DEFAULT '',"
                . " `OXPOS` int(11) NOT NULL DEFAULT '0',"
                . "PRIMARY KEY (`OXID`),"
                . "KEY `OXOBJECTID` (`OXOBJECTID`),"
                . "KEY `OXMAINIDX` (`OXCATNID`,`OXOBJECTID`));";
        $sqlStatements['create_assigned_manufacturers'] = "CREATE TABLE IF NOT EXISTS `mo_viaebay__assigned_manufacturers` ("
                . "`OXID` char(32) CHARACTER SET latin1 COLLATE latin1_general_ci NOT NULL,"
                . "`OXOBJECTID` char(32) CHARACTER SET latin1 COLLATE latin1_general_ci NOT NULL DEFAULT '',"
                . "`OXMANUFACTURERID` char(32) CHARACTER SET latin1 COLLATE latin1_general_ci NOT NULL DEFAULT '',"
                . "`OXPOS` int(11) NOT NULL DEFAULT '0',"
                . "PRIMARY KEY (`OXID`),"
                . "KEY `OXOBJECTID` (`OXOBJECTID`),"
                . "KEY `OXMAINIDX` (`OXMANUFACTURERID`,`OXOBJECTID`));";
        $sqlStatements['create_order_details'] = "CREATE TABLE IF NOT EXISTS `mo_viaebay__order_details` ("
                . "`OXID` CHAR( 32 ) CHARACTER SET latin1 COLLATE latin1_general_ci NOT NULL ,"
                . "`MO_VIAEBAY__VIA_ID` INT NOT NULL ,"
                . "`MO_VIAEBAY__ORDER_DETAILS` TEXT NOT NULL ,"
                . "`MO_VIAEBAY__HISTORY` TEXT NOT NULL ,"
                . "PRIMARY KEY ( `OXID` ) ,"
                . "UNIQUE (`MO_VIAEBAY__VIA_ID`))"
                . "ENGINE = MYISAM ;";
        $sqlStatements['updateOxOrder_SHIPPING_STATE'] = "ALTER TABLE `oxorder` ADD `MO_VIAEBAY__SHIPPING_STATE` TINYINT( 1 ) NULL;";
        $sqlStatements['updateOxOrder_PAYMENT_STATE'] = "ALTER TABLE `oxorder` ADD `MO_VIAEBAY__PAYMENT_STATE` TINYINT( 1 ) NULL;";
        $sqlStatements['OxConfigentry1'] = "INSERT INTO oxconfig SET "
                . "OXID = 'mo_viaebay__imageSizes', "
                . "OXSHOPID = 'oxbaseshop', "
                . "OXMODULE = 'mo_viaebay', "
                . "OXVARNAME = 'aDetailImageSizes', "
                . "OXVARTYPE = 'arr', "
                . "OXVARVALUE = ENCODE('a:2:{i:0;s:7:\"225*225\";i:1;s:9:\"1500*1500\";}', 'fq45QS09_fqyx09239QQ');";
        $sqlStatements['OxConfigentry2'] = "INSERT INTO oxconfig SET "
                . "OXID = 'mo_viaebay__thumbSize', "
                . "OXSHOPID = 'oxbaseshop', "
                . "OXMODULE = 'mo_viaebay', "
                . "OXVARNAME = 'sThumbnailsize', "
                . "OXVARTYPE = 'str', "
                . "OXVARVALUE = ENCODE('225*225', 'fq45QS09_fqyx09239QQ');";
        $sqlStatements['updateOxArtExtends'] = "ALTER TABLE  `oxartextends` ADD  `MO_VIAEBAY__LONG_DESC` TEXT CHARACTER SET utf8 COLLATE utf8_general_ci NOT NULL;";
        $sqlStatements['create_articles2via'] = "CREATE TABLE IF NOT EXISTS `mo_viaebay__articles2via` ( "
                . "`OXID` char(32) COLLATE latin1_general_ci NOT NULL, "
                . "`OXSHOPID` varchar(32) NOT NULL, "
                . "`OXARTID` char(32) COLLATE latin1_general_ci NOT NULL, "
                . "`MO_VIAEBAY__VIA_ID` int(11) NOT NULL, "
                . "`MO_VIAEBAY__LAST_SYNC` datetime NOT NULL, "
                . "`MO_VIAEBAY__ALLOW_EXPORT` BOOLEAN NOT NULL DEFAULT '1', "
                . "PRIMARY KEY (`OXID`) "
                . ") ENGINE=InnoDB DEFAULT CHARSET=latin1 COLLATE=latin1_general_ci;";
        $sqlStatements['create_category2via'] = "CREATE TABLE IF NOT EXISTS `mo_viaebay__category2via` ( "
                . "`OXID` char(32) COLLATE latin1_general_ci NOT NULL, "
                . "`OXSHOPID` varchar(32) NOT NULL, "
                . "`OXCATNID` char(32) COLLATE latin1_general_ci NOT NULL, "
                . "`MO_VIAEBAY__VIA_ID` int(11) DEFAULT NULL, "
                . "`MO_VIAEBAY__VIA_PARENT_ID` int(11) DEFAULT NULL, "
                . "`MO_VIAEBAY__LAST_SYNC` datetime NOT NULL, "
                . "`OXTIMESTAMP` timestamp NOT NULL DEFAULT CURRENT_TIMESTAMP "
                . ") ENGINE=InnoDB DEFAULT CHARSET=latin1 COLLATE=latin1_general_ci;";


        return $sqlStatements;
    }

    protected static function getArtExtendUpdateSqlStatements($sqlStatements)
    {
        $sqlStatements['updateOxArtExtends'] = "ALTER TABLE  `oxartextends` ADD  `MO_VIAEBAY__LONG_DESC` TEXT CHARACTER SET utf8 COLLATE utf8_general_ci NOT NULL;";
        return $sqlStatements;
    }

    protected static function getMultishopUpdateSqlStatements()
    {
        $sqlStatements['create_articles2via'] = "CREATE TABLE IF NOT EXISTS `mo_viaebay__articles2via` ( "
                . "`OXID` char(32) COLLATE latin1_general_ci NOT NULL, "
                . "`OXSHOPID` varchar(32) NOT NULL, "
                . "`OXARTID` char(32) COLLATE latin1_general_ci NOT NULL, "
                . "`MO_VIAEBAY__VIA_ID` int(11) NOT NULL, "
                . "`MO_VIAEBAY__LAST_SYNC` datetime NOT NULL, "
                . "`MO_VIAEBAY__ALLOW_EXPORT` tinyint(1) NOT NULL DEFAULT '1', "
                . "PRIMARY KEY (`OXID`) "
                . ") ENGINE=InnoDB DEFAULT CHARSET=latin1 COLLATE=latin1_general_ci;";

        $sqlStatements['create_category2via'] = "CREATE TABLE IF NOT EXISTS `mo_viaebay__category2via` ( "
                . "`OXID` char(32) COLLATE latin1_general_ci NOT NULL, "
                . "`OXSHOPID` varchar(32) NOT NULL, "
                . "`OXCATNID` char(32) COLLATE latin1_general_ci NOT NULL, "
                . "`MO_VIAEBAY__VIA_ID` int(11) DEFAULT NULL, "
                . "`MO_VIAEBAY__VIA_PARENT_ID` int(11) DEFAULT NULL, "
                . "`MO_VIAEBAY__LAST_SYNC` datetime NOT NULL, "
                . "`OXTIMESTAMP` timestamp NOT NULL DEFAULT CURRENT_TIMESTAMP "
                . ") ENGINE=InnoDB DEFAULT CHARSET=latin1 COLLATE=latin1_general_ci;";

        $sqlStatements['disableArticleKeys'] = "ALTER TABLE `oxarticles` DISABLE KEYS; ";
        $sqlStatements['copyArticlesViaData'] = "INSERT INTO mo_viaebay__articles2via (`oxid`,`oxshopid`,`oxartid`,`mo_viaebay__via_id`,`mo_viaebay__last_sync`,`MO_VIAEBAY__ALLOW_EXPORT`) "
                . "SELECT md5(`oxid`),`oxshopid`,`oxid`, `mo_viaebay__via_id`,`mo_viaebay__last_sync`,`MO_VIAEBAY__ALLOW_EXPORT` FROM oxarticles "
                . "WHERE oxarticles.MO_VIAEBAY__VIA_ID IS NOT NULL; ";
        $sqlStatements['enableArticleKeys'] = "ALTER TABLE `oxarticles` ENABLE KEYS; ";

        $sqlStatements['disableCategoriesKeys'] = "ALTER TABLE `oxcategories` DISABLE KEYS; ";
        $sqlStatements['copyCategoryViaData'] = "INSERT INTO mo_viaebay__category2via (`oxid`,`oxshopid`,`oxcatnid`,`mo_viaebay__via_id`,`mo_viaebay__via_parent_id`, `mo_viaebay__last_sync`) "
                . "SELECT md5(`oxid`),`oxshopid`,`oxid`, `mo_viaebay__via_id`,`mo_viaebay__via_parent_id`,`mo_viaebay__last_sync` FROM oxcategories "
                . "WHERE oxcategories.MO_VIAEBAY__VIA_ID IS NOT NULL; ";
        $sqlStatements['enableCategoriesKeys'] = "ALTER TABLE `oxcategories` ENABLE KEYS; ";

        $sqlStatements['dropOxArtAttributes'] = "ALTER TABLE `oxarticles` DROP `MO_VIAEBAY__VIA_ID`, DROP `MO_VIAEBAY__LAST_SYNC`, DROP `MO_VIAEBAY__ALLOW_EXPORT`;";
        $sqlStatements['dropOxCatAttributes'] = "ALTER TABLE `oxcategories` DROP `MO_VIAEBAY__VIA_ID`, "
                . "DROP `MO_VIAEBAY__VIA_PARENT_ID`, "
                . "DROP `MO_VIAEBAY__LAST_SYNC`;";


        return $sqlStatements;
    }

    /**
     * Since Version 1.3.1 oxpayments has an  mo_viaebay-entry
     * @return boolean
     */
    protected static function isVersionUnder131()
    {
        $dbName = oxRegistry::get("oxConfigFile")->getVar('dbName');
        $oxpaymentsColumns = oxDb::getDb()->Execute("SHOW COLUMNS FROM `oxpayments` FROM `"
                . $dbName . "`  LIKE '%mo_viaebay%'");
        if ($oxpaymentsColumns->RecordCount() == 0) {
            return true;
        }
        return false;
    }

    protected static function get131DBUpdate($sqlStatements)
    {
        $sqlStatements['updateoxattribute_ATTRIBUTE_NAME'] = "ALTER TABLE  `oxattribute` ADD  `MO_VIAEBAY__ATTRIBUTE_NAME` "
                . "CHAR( 32 ) CHARACTER SET latin1 COLLATE latin1_general_ci NULL DEFAULT NULL";
        $sqlStatements['updateoxattribute_ATTRIBUTES_AS'] = "ALTER TABLE  `oxattribute` ADD  `MO_VIAEBAY__ATTRIBUTES_AS` INT( 11 ) NOT NULL DEFAULT  '0'";
        $sqlStatements['updateoxoxpayments_PAYMENT_NAME'] = "ALTER TABLE  `oxpayments` ADD  `MO_VIAEBAY__PAYMENT_NAME` "
                . "CHAR( 32 ) CHARACTER SET latin1 COLLATE latin1_general_ci NULL";
        $sqlStatements['updateoxdeliveryset_SHIPPING_NAME'] = "ALTER TABLE  `oxdeliveryset` ADD  `MO_VIAEBAY__SHIPPING_NAME` "
                . "CHAR( 32 ) CHARACTER SET latin1 COLLATE latin1_general_ci NULL DEFAULT NULL";
        return $sqlStatements;
    }

    protected static function checkArticles2ViaIndex()
    {
        $oxDb = oxDb::getDb();
        $sql = "SHOW INDEX FROM `mo_viaebay__articles2via` WHERE Column_name = 'OXARTID'";

        if (!$oxDb->Execute($sql)->RecordCount()) {
            $sql = "ALTER TABLE `mo_viaebay__articles2via` ADD INDEX(`OXARTID`)";
            $oxDb->Execute($sql);
        }
    }

    protected static function checkCategory2ViaIndex()
    {
        $oxDb = oxDb::getDb();
        $sql = "SHOW INDEX FROM `mo_viaebay__category2via` WHERE Column_name = 'OXCATNID'";

        if (!$oxDb->Execute($sql)->RecordCount()) {
            $sql = "ALTER TABLE `mo_viaebay__category2via` ADD (INDEX(`OXCATNID`) , PRIMARY KEY (`OXID`)); ";
            $oxDb->Execute($sql);
        }
    }

    protected static function addTrackingSyncColumn()
    {
        $oxDb = oxDb::getDb();
        $result = $oxDb->execute("SHOW COLUMNS FROM `mo_viaebay__order_details` LIKE 'MO_VIAEBAY__TRACKING_CODE'");
        if (!$result->RecordCount()) {
            $sql = "ALTER TABLE mo_viaebay__order_details ADD MO_VIAEBAY__TRACKING_CODE VARCHAR(128) COLLATE utf8_general_ci NOT NULL";
            $oxDb->execute($sql);
        }
    }
    
    protected static function addInventorySyncDateColumn()
    {
        $oxDb = oxDb::getDb();
        $result = $oxDb->execute("SHOW COLUMNS FROM `mo_viaebay__articles2via` LIKE 'MO_VIAEBAY__LAST_MPSYNC'");
        if (!$result->RecordCount()) {
            $sql = "ALTER TABLE mo_viaebay__articles2via ADD MO_VIAEBAY__LAST_MPSYNC DATETIME NOT NULL";
            $oxDb->execute($sql);
        }
    }
}
