<?php

/**
 * mo_viaebay__interface
 * 
 * delegate API communication
 * 
 * @version   $Id: mo_viaebay__interface.php 161 2012-12-17 13:20:04Z $
 */

/**
 * delegate API communication
 * 
 * @package   mo_viaebay 
 */
class mo_viaebay__interface
{

    /**
     * store logger
     *
     * @var mo_viaebay__logger
     */
    protected $logger;

    /**
     * store client
     *
     * @var mo_viaebay__client 
     */
    protected $client;

    /**
     * store feedback handler
     *
     * @var mo_viaebay__feedbackHandler 
     */
    protected $feedbackHandler;

    /**
     * store param builder
     *
     * @var mo_viaebay__paramBuilder 
     */
    protected $paramBuilder;

    /**
     * constructor
     * 
     * @param mo_viaebay__client $client
     * @param mo_viaebay__feedback_handler $feedbackHandler
     * @param mo_viaebay__param_builder $paramBuilder
     * @param mo_viaebay__logger $logger 
     */
    public function __construct(
    mo_viaebay__client $client, mo_viaebay__feedback_handler $feedbackHandler, mo_viaebay__param_builder $paramBuilder, mo_viaebay__logger $logger)
    {
        $this->client = $client;
        $this->feedbackHandler = $feedbackHandler;
        $this->paramBuilder = $paramBuilder;
        $this->logger = $logger;
    }

    /**
     * generic call
     * 
     * @param string $fnc
     * @param mixed $args
     * @return array 
     */
    protected function execute($fnc, $args)
    {
        $this->logger->debug($fnc);
        $fnc = ucfirst($fnc);
        $fbFnc = 'handle' . $fnc;
        $params = call_user_func_array(array($this->paramBuilder, 'build' . $fnc), $args);
        $response = $this->client->callService($params);
        try {
            $result = $this->feedbackHandler->$fbFnc($response);
        } catch (oxConnectionException $e) {
            $this->logger->debug('Auth failure. Renew session cookie...');
            $this->getSessionCookie();
            $params = $this->paramBuilder->updateAuthParameters($params);
            $response = $this->client->callService($params);
            $result = $this->feedbackHandler->$fbFnc($response);
        }
        return $result;
    }

    /**
     * get session cookie
     *
     * @return mixed 
     */
    public function getSessionCookie()
    {
        return $this->execute(__FUNCTION__, array());
    }

    /**
     * get categories
     *
     * @param array $ids
     * @return mixed 
     */
    public function getCategories($ids = array())
    {
        return $this->execute(__FUNCTION__, array($ids));
    }

    /**
     * add category
     *
     * @param oxCategory $category
     * @return mixed 
     */
    public function addCategory(oxCategory $category)
    {
        return $this->execute(__FUNCTION__, array($category));
    }

    /**
     * update category
     *
     * @param oxCategory $category
     * @return mixed 
     */
    public function updateCategory(oxCategory $category)
    {
        return $this->execute(__FUNCTION__, array($category));
    }

    /**
     * delete category
     *
     * @param string $categoryViaId
     * @return mixed 
     */
    public function deleteCategory($categoryViaId)
    {
        return $this->execute(__FUNCTION__, array($categoryViaId));
    }

    /**
     * get category relations
     *
     * @param string $categoryViaId
     * @return mixed 
     */
    public function getCategoryRelations($categoryViaId)
    {
        return $this->execute(__FUNCTION__, array($categoryViaId));
    }

    /**
     * set category relation
     *
     * @param string $parentCategoryViaId
     * @param string $subCategoryViaId
     * @return mixed 
     */
    public function setCategoryRelation($parentCategoryViaId, $subCategoryViaId)
    {
        return $this->execute(__FUNCTION__, array($parentCategoryViaId, $subCategoryViaId));
    }

    /**
     * delete category relation
     *
     * @param string $parentCategoryViaId
     * @param string $subCategoryViaId
     * @return mixed 
     */
    public function deleteCategoryRelation($parentCategoryViaId, $subCategoryViaId)
    {
        return $this->execute(__FUNCTION__, array($parentCategoryViaId, $subCategoryViaId));
    }

    /**
     * get articles
     *
     * @param array $ids
     * @return mixed 
     */
    public function getArticles($ids = array(), $options = array())
    {
        return $this->execute(__FUNCTION__, array($ids, $options));
    }

    /**
     * get article ids
     *
     * @return mixed 
     */
    public function getArticleIds()
    {
        return $this->execute(__FUNCTION__, array());
    }

    /**
     * add article
     *
     * @param oxArticle $article
     * @return mixed 
     */
    public function addArticle(oxArticle $article)
    {
        return $this->execute(__FUNCTION__, array($article));
    }

    /**
     * dlete article
     *
     * @param string $articleViaId
     * @return mixed 
     */
    public function deleteArticle($articleViaId)
    {
        return $this->execute(__FUNCTION__, array($articleViaId));
    }

    /**
     * update article
     *
     * @param oxArticle $article
     * @return mixed 
     */
    public function updateArticle(oxArticle $article)
    {
        return $this->execute(__FUNCTION__, array($article));
    }

    /**
     * yadd article image
     *
     * @param mixed $imageData
     * @return mixed 
     */
    public function addArticleImage($imageData)
    {
        return $this->execute(__FUNCTION__, array($imageData));
    }

    /**
     * delete article image
     *
     * @param string $viaId
     * @return mixed 
     */
    public function deleteArticleImage($viaId)
    {
        return $this->execute(__FUNCTION__, array($viaId));
    }

    /**
     * add article category relation
     *
     * @param string $viaArticleId
     * @param string $viaCategoryId
     * @return mixed 
     */
    public function addArticleCategoryRelation($viaArticleId, $viaCategoryId)
    {
        return $this->execute(__FUNCTION__, array($viaArticleId, $viaCategoryId));
    }

    /**
     * delete article category relation
     *
     * @param string $viaArticleId
     * @param string $viaCategoryId
     * @return mixed 
     */
    public function deleteArticleCategoryRelation($viaArticleId, $viaCategoryId)
    {
        return $this->execute(__FUNCTION__, array($viaArticleId, $viaCategoryId));
    }

    /**
     * add article attribute
     *
     * @param string $viaArticleId
     * @param string $attributeName
     * @param string $attributeValue
     * @return mixed 
     */
    public function addArticleAttribute($viaArticleId, $attributeName, $attributeValue)
    {
        return $this->execute(__FUNCTION__, array($viaArticleId, $attributeName, $attributeValue));
    }

    /**
     * add article attribute
     *
     * @param string $viaAttributeId
     * @param string $viaArticleId
     * @param string $attributeName
     * @param string $attributeValue
     * @return mixed 
     */
    public function updateArticleAttribute($viaAttributeId, $viaArticleId, $attributeName, $attributeValue)
    {
        return $this->execute(__FUNCTION__, array($viaAttributeId, $viaArticleId, $attributeName, $attributeValue));
    }

    /**
     * delete article attribute
     *
     * @param string $viaAttributeId
     * @return mixed 
     */
    public function deleteArticleAttribute($viaAttributeId)
    {
        return $this->execute(__FUNCTION__, array($viaAttributeId));
    }

    /**
     * add Additional Information to article object 
     *
     * @param string $viaArticleId
     * @param string $attributeName
     * @param string $attributeValue
     * @return mixed 
     */
    public function addAdditionalInformation($viaArticleId, $attributeName, $attributeValue)
    {
        return $this->execute(__FUNCTION__, array($viaArticleId, $attributeName, $attributeValue));
    }

    /**
     * add Additional Information to article object 
     *
     * @param string $viaArticleId
     * @param string $attributeName
     * @param string $attributeValue
     * @return mixed 
     */
    public function updateAdditionalInformation($viaAttributeId, $viaArticleId, $attributeName, $attributeValue)
    {
        return $this->execute(__FUNCTION__, array($viaAttributeId, $viaArticleId, $attributeName, $attributeValue));
    }

    /**
     * delete additional Information
     *
     * @param string $viaAttributeId
     * @return mixed 
     */
    public function deleteAdditionalInformation($viaAttributeId)
    {
        return $this->execute(__FUNCTION__, array($viaAttributeId));
    }

    /**
     * add variant as ProductVariation
     * 
     * @param int $viaArticleId
     * @param oxArticle $variant
     * @return type 
     */
    public function addArticleVariant($viaArticleId, oxArticle $variant)
    {
        return $this->execute(__FUNCTION__, array($viaArticleId, $variant));
    }

    /**
     * update variant
     * @param oxArticle $oxArticleVariant
     * @return boolean 
     */
    public function updateArticleVariant(oxArticle $oxArticleVariant)
    {
        return $this->execute(__FUNCTION__, array($oxArticleVariant));
    }

    /**
     * delete variant
     * @param int $viaId
     * @return boolean
     */
    public function deleteArticleVariant($viaVariant)
    {
        return $this->execute(__FUNCTION__, array($viaVariant));
    }

    /**
     * ad variant specific
     * @param int $viaId
     * @param string $name
     * @param string $value
     * @return boolean 
     */
    public function addArticleVariantSpecific($viaId, $name, $value)
    {
        return $this->execute(__FUNCTION__, array($viaId, $name, $value));
    }

    /**
     * delete article variant
     * @param int $viaId
     * @return boolean 
     */
    public function deleteArticleVariantSpecific($viaId)
    {
        return $this->execute(__FUNCTION__, array($viaId));
    }

    /**
     * get orders
     *
     * @param array $ids
     * @return mixed 
     */
    public function getOrders($ids = array())
    {
        return $this->execute(__FUNCTION__, array($ids));
    }

    /**
     * update order
     *
     * @param string $viaId
     * @param string $propertyName
     * @param string $propertyValue
     * @return mixed 
     */
    public function updateOrder($viaId, $propertyName, $propertyValue)
    {
        return $this->execute(__FUNCTION__, array($viaId, $propertyName, $propertyValue));
    }

    /**
     * report payment
     *
     * @param oxOrder $order
     * @return mixed 
     */
    public function reportPayment(oxOrder $order)
    {
        return $this->execute(__FUNCTION__, array($order));
    }

    /**
     * report shipment
     *
     * @param oxOrder $order
     * @return mixed 
     */
    public function reportShipment(oxOrder $order)
    {
        return $this->execute(__FUNCTION__, array($order));
    }

    /**
     * report Trackingcode
     * 
     * @param oxOrder $trackCode
     * @return type
     */
    public function reportTrackingcode(oxOrderArticle $article, $trackCode)
    {
        return $this->execute(__FUNCTION__, array($article, $trackCode));
    }
    
    /**
     * report CarrierUsed
     * 
     * @param oxOrder $order
     * @return type
     */
    public function reportCarrierUsed(oxOrderArticle $article, $carrier)
    {
        return $this->execute(__FUNCTION__, array($article, $carrier));
    }

    /**
     * set profile for product
     * 
     * @param type $productId
     * @param type $profileId
     * @param type $type
     * @return type
     */
    public function setProductProfile($productId, $profileId, $type)
    {
        return $this->execute(__FUNCTION__, array($productId, $profileId, $type));
    }

    /**
     * update stock & price
     *
     * @param oxArticle $oxArticle
     * @return mixed 
     */
    public function updateInventory(oxArticle $oxArticle)
    {
        return $this->execute(__FUNCTION__, array($oxArticle));
    }

}
