<?php

/**
 * mo_viaebay__logger
 * 
 * file-logger
 * 
 * @version   $Id: mo_viaebay__logger.php 161 2012-12-17 13:20:04Z $
 */

/**
 * file-logger
 * 
 * @package   mo_viaebay 
 */
class mo_viaebay__logger
{

  /**
   * log stack-trace & message
   * 
   * @param string $message 
   */
  public function logExecution($message = '', $prefix = '')
  {
    if (!is_string($message))
    {
      $message = var_export($message, true);
    }
    if (!empty($message))
    {
      $message   = ' ' . $message;
    }
    
    $this->message('DEBUG', $prefix . $message);
  }

  /**
   * log info-level
   * 
   * @param string $message 
   */
  public function info($message)
  {
    $this->outputMessage($message, 'INFO');
    $this->message('INFO', $message);
  }

  /**
   * log debug-level
   * 
   * @param string $message 
   */
  public function debug($message)
  {
    $this->message('DEBUG', $message);
  }
  
  /**
   * log error level
   * 
   * @param string $message
   * @param mixed $infoObject 
   */
  public function error($message, $infoObject = null)
  {
    if ($infoObject)
    {
      $message .= "\n" . print_r($infoObject, true) . "\n" . str_repeat('-', 80) . "\n";
    }
    $this->outputMessage($message, 'ERROR');
    $this->message('ERROR', $message);
  }

  /**
   * write into file
   * 
   * @param string $type
   * @param string $message 
   */
  public function message($type, $message)
  {
    $logFile = oxRegistry::getConfig()->getLogsDir() . 'mo_viaebay.log';

    $this->logRotation($logFile);

    $date = date("Y-m-d H:i:s", time());

    $fh = fopen($logFile, "a");
    if ($fh)
    {
      $this->write($fh, "$date $type: $message \n");
      fclose($fh);
    }
  }

  /**
   * output message when called in script context
   *
   * @param string $message
   * @param string $level
   * @return mixed 
   */
  protected function outputMessage($message, $level)
  {
    if (!defined('MO_VIAEBAY__IS_SCRIPT_CONTEXT'))
    {
      return;
    }
    
    if (defined('MO_VIAEBAY__OUTPUT_LOG') && !MO_VIAEBAY__OUTPUT_LOG) {
        return;
    }
    
    $output = $level . ': ' . $message;
    if(PHP_SAPI == 'cli') {
        echo $output . "\n";
    } else {
        echo '<pre>' . htmlentities($output) . '</pre>';
        flush();
        ob_end_flush();
    }
  }

  /**
   * write into file (extracted for unit tests)
   *
   * @param fileHandler $fh
   * @param string $string
   */
  protected function write($fh, $string)
  {
    fwrite($fh, $string);
  }

  /**
   * backup logfiles (maybe implement "real" rotation later)
   * 
   * @param string $file
   */
  protected function logRotation($file)
  {
    if ($this->isRotationActive($file))
    {
      $backupLogFile = preg_replace('#^(.*)?\.([a-z]+)$#', '$1_' . time() . '_' . uniqid() . '.$2.zip', strtolower($file));

      $zip = oxNew("ZipArchive");
      $zip->open($backupLogFile, ZipArchive::CREATE);
      $zip->addFile($file, preg_replace('#^.*?/([^/]+)$#', '$1', $file));
      $zip->close();

      unlink($file);
    }
  }

  /**
   * check if log rotation is active
   * 
   * @param string $file
   * @return boolean
   */
  protected function isRotationActive($file)
  {
    return file_exists($file) && class_exists('ZipArchive') && filesize($file) > 10000000; //~10MB
  }

}

