<?php

/**
 * mo_viaebay__main
 * 
 * main singleton wiring class 
 * 
 * @version   $Id: mo_viaebay__main.php 220 2013-06-06 14:06:26Z $
 */

/**
 * main singleton wiring class
 * 
 * @package   mo_viaebay  
 */
class mo_viaebay__main
{

  /**
   * access instance of main class
   *
   * @staticvar mo_viaebay__main 
   */
  static protected $instance = null;

  /**
   * via ebay configuration
   *
   * @var array 
   */
  protected $viaebayConfig = null;

  /**
   * instance of oxConfig
   *
   * @var oxconfig 
   */
  protected $oxConfig = null;

  /**
   * instance of oxSession
   *
   * @var oxsession 
   */
  protected $oxSession = null;

  /**
   * logger
   *
   * @var mo_viaebay__logger
   */
  protected $logger = null;

  /**
   * param builder
   *
   * @var mo_viaebay__param_builder
   */
  protected $paramBuilder = null;

  /**
   * feedback handler
   *
   * @var mo_viaebay__feedback_handler 
   */
  protected $feedbackHandler = null;

  /**
   * client
   *
   * @var mo_viaebay__client 
   */
  protected $client = null;

  /**
   * interface
   *
   * @var mo_viaebay__interface 
   */
  protected $interface = null;

  /**
   * task manager
   *
   * @var mo_viaebay__task_manager 
   */
  protected $taskManager = null;

  /**
   * helper
   *
   * @var mo_viaebay__helper 
   */
  protected $helper = null;

  /**
   * @staticvar string 
   */

  const MO_VIAEBAY__PRODUCT_TYPE__SINGLE_PRODUCT = 0;

  /**
   * @staticvar string 
   */
  const MO_VIAEBAY__PRODUCT_TYPE__VARIATION_SET = 1;
  /**
   * @staticvar string 
   */
  const MO_VIAEBAY__PRODUCT_TYPE__PRODUCT_SET   = 2;
  /**
   * @staticvar string 
   */
  const MO_VIAEBAY__IMAGE_TYPE__STANDARD        = 2;
  /**
   * @staticvar string 
   */
  const MO_VIAEBAY__IMAGE_TYPE__LISTING       = 1;
  /**
   * @staticvar string 
   */
  const MO_VIAEBAY__CRON_CONTROL_IS_ACTIVE      = 1;
  /**
   * @staticvar int
   */
  const MO_VIAEBAY__PROFILE_ID__PAYMENT = 1;
  /**
   * @staticvar int
   */
  const MO_VIAEBAY__PROFILE_ID__RETURN = 2;
  /**
   * @staticvar int
   */
  const MO_VIAEBAY__PROFILE_ID__SHIPPING = 3;
  
  const MO_VIAEBAY__TICK_SIZE = 20;
  
  /**
   * request format
   *
   * @staticvar array 
   */
  static public $REQUEST_FORMAT__JSON = array(
      'ident'        => 'json',
      'content_type' => 'application/json',
  );

  /**
   * avaiable references, user for ajax assigment
   *
   * @var array 
   */
  protected $mo_viaebay__reference = array(
      'viaebay_oxcategories' => array(
          'class'                 => 'oxcategory',
          'table'                 => 'oxcategories',
          'display_column'        => 'oxtitle',
          'ref_table'             => 'mo_viaebay__assigned_categories',
          'ref_field_1'           => 'oxobjectid', // current edit object
          'ref_field_2'           => 'oxcatnid', // referenced objects
          'lang_key'              => 'VIAEBAY_OXCATEGORIES',
          'sort_column'           => 'oxpos',
          'ref_table_has_shop_id' => false,
      ),
      'viaebay_manufacturers' => array(
          'class'                 => 'oxmanufacturer',
          'table'                 => 'oxmanufacturers',
          'display_column'        => 'oxtitle',
          'ref_table'             => 'mo_viaebay__assigned_manufacturers',
          'ref_field_1'           => 'oxobjectid', // current edit object
          'ref_field_2'           => 'oxmanufacturerid', // referenced objects
          'lang_key'              => 'VIAEBAY_MANUFACTURERS',
          'sort_column'           => 'oxpos',
          'ref_table_has_shop_id' => false,
      ),
  );

  /**
   * available tasks
   *
   * @var array 
   */
  protected $mo_viaebay__tasks = array(
      'pull_orders'     => 'VIA-eBay Bestellungen:',
      'sync_categories' => 'Shop Kategorien:',
      'sync_articles'   => 'Artikeldaten:',
      'sync_inventory'  => 'Artikel Preis/Menge:',
  );

  /**
   * empty config, shows all config parameters and structure
   *
   * @var array 
   */
  protected $mo_viaebay__configSkeleton = array(
      'api_username'       => '',
      'api_password'       => '',
      'subscription_token' => '',
      'stock_export'       => '',
      'export_language'    => '',
      'is_sandbox'         => '',
      //key: task, value: sync date
      'tasks'              => array(
          'sync_categories' => array(
              'interval'       => '',
              'last_execution' => '',
          ),
          'sync_articles'  => array(
              'interval'       => '',
              'last_execution' => '',
          ),
          'pull_orders'    => array(
              'interval'           => '',
              'last_execution'     => '',
          ),
          'sync_inventory'    => array(
              'interval'           => '',
              'last_execution'     => '',
          ),
      ),
      'mo_viaebay__filter' => array(
          'min_price'     => '',
          'max_price'     => '',
          'article_image' => '',
          'min_stock'     => '',
      ),
  );

  /**
   * check if module is activated
   * 
   * @return boolean 
   */
  public function isModuleActivated()
  {
    $config = $this->getViaebayConfig();
    if (empty($config['is_active']))
    {
      return false;
    }
    return true;
  }

  /**
   * singleton accessor
   * 
   * @return mo_viaebay__main 
   */
  static public function getInstance()
  {
    if (is_null(self::$instance))
    {
      self::$instance = oxNew('mo_viaebay__main');
    }
    return self::$instance;
  }

  /**
   * mock instance for unit-tests
   * 
   * @param object $mock 
   */
  static public function unitTestSetMockInstance($mock)
  {
    self::$instance = $mock;
  }

  /**
   * remove mock instance
   */
  static public function unitTestRemoveMockInstance()
  {
    self::$instance = null;
  }

  /**
   * constructor
   */
  public function __construct()
  {
    $this->oxConfig = oxRegistry::getConfig();
    $this->oxSession = oxRegistry::getSession();
    $this->viaebayConfig = $this->oxConfig->getShopConfVar('mo_viaebay__config');
  }

  /**
   * viaebay config getter
   * 
   * @return array viaebay config
   */
  public function getViaebayConfig()
  {
    if ($this->viaebayConfig !== null)
    {
      return $this->viaebayConfig;
    }

    if (!$this->viaebayConfig = $this->oxConfig->getShopConfVar('mo_viaebay__config'))
    {
      $this->viaebayConfig = $this->mo_viaebay__configSkeleton;
    }

    return $this->viaebayConfig;
  }

  /**
   * set viaebay config
   *
   * @param array $config 
   */
  public function setViaebayConfig($config)
  {
    $this->viaebayConfig = $config;
  }

  /**
   * save via ebay config 
   */
  public function saveViaConfig()
  {
    $this->oxConfig->saveShopConfVar('arr', 'mo_viaebay__config', $this->viaebayConfig);
  }

  /**
   * viaebay reference getter
   * 
   * @return array viaebay reference
   */
  public function getViaebayReference()
  {
    return $this->mo_viaebay__reference;
  }

  /**
   * logger getter
   * 
   * @return mo_viaebay__logger 
   */
  public function getLogger()
  {
    if (is_null($this->logger))
    {
      $this->logger = oxNew('mo_viaebay__logger');
    }
    return $this->logger;
  }

  /**
   * param builder getter
   * 
   * @return mo_viaebay__param_builder 
   */
  public function getParamBuilder()
  {
    if (is_null($this->paramBuilder))
    {
      $this->paramBuilder = oxNew('mo_viaebay__param_builder', $this->oxConfig, $this->oxSession, $this->getViaebayConfig(), $this->getLogger());
    }
    return $this->paramBuilder;
  }

  /**
   * getter method for feedback handler
   * 
   * @return mo_viaebay__feedback_handler 
   */
  public function getFeedbackHandler()
  {
    if (is_null($this->feedbackHandler))
    {
      $this->feedbackHandler = oxNew('mo_viaebay__feedback_handler', $this->oxConfig, $this->oxSession, $this->getViaebayConfig(), $this->getLogger());
    }
    return $this->feedbackHandler;
  }

  /**
   * client getter
   * 
   * @return mo_viaebay__client 
   */
  public function getClient()
  {
    if (is_null($this->client))
    {
      $this->client = oxNew('mo_viaebay__client', $this->oxSession, $this->getLogger());
    }
    return $this->client;
  }

  /**
   * getter for interface
   * 
   * @return mo_viaebay__interface 
   */
  public function getInterface()
  {
    if (is_null($this->interface))
    {
      $this->interface = oxNew('mo_viaebay__interface', $this->getClient(), $this->getFeedbackHandler(), $this->getParamBuilder(), $this->getLogger());
    }
    return $this->interface;
  }

  /**
   * get sync manager
   *
   * @param string $type
   * @param array $ids
   * @return mo_viaebay__sync_manager 
   */
  public function getSyncManager($type, $ids = array())
  {
    $class = 'mo_viaebay__sync_manager_' . $type;
    return $this->syncManagers[$type] = oxNew($class, $this->getFeed($type, $ids), $this->getParamBuilder(), $this->getLogger(), $this->getHelper(), $this->getViaebayConfig(), $this->oxConfig);
  }

  /**
   * get feed
   *
   * @param string $type
   * @param array $ids
   * @return mo_viaebay__feed 
   */
  public function getFeed($type, $ids = array(), $options = array())
  {
    $class = 'mo_viaebay__feed_' . $type;
    return $this->feeds[$type] = oxNew($class, $ids, $options);
  }

  /**
   * get tasks
   *
   * @return array 
   */
  public function getTasks()
  {
    return $this->mo_viaebay__tasks;
  }

  /**
   * get taskmanager
   *
   * @return mo_viaebay__task_manager 
   */
  public function getTaskManager()
  {
    if (is_null($this->taskManager))
    {
      $this->taskManager = oxNew('mo_viaebay__task_manager', $this->getViaebayConfig());
    }
    return $this->taskManager;
  }

  /**
   * get helper
   *
   * @return mo_viaebay__helper 
   */
  public function getHelper()
  {
    if (is_null($this->helper))
    {
      $this->helper = oxNew('mo_viaebay__helper');
    }
    return $this->helper;
  }

}