<?php

/**
 * mo_viaebay__param_builder
 * 
 * collects necessary data for API-calls
 * 
 * @version   $Id: mo_viaebay__param_builder.php 218 2013-06-05 11:37:37Z $
 */

/**
 * collects necessary data for API-calls
 * 
 * @package   mo_viaebay
 */
class mo_viaebay__param_builder
{

    /**
     * @staticvar 
     */
    const SERVICE_URL__DEV = 'https://sandboxapi.via.de/publicapi/v1/api.svc/';

    /**
     * @staticvar 
     */
    const SERVICE_URL__PROD = 'https://ebayapi.via.de/publicapi/v1/api.svc/';

    /**
     * @staticvar 
     */
    const AUTH_URL__DEV = 'https://sandboxapi.via.de/Authentication_JSON_AppService.axd/';

    /**
     * @staticvar 
     */
    const AUTH_URL__PROD = 'https://ebayapi.via.de/Authentication_JSON_AppService.axd/';

    /**
     * return live or sandbox service-url depending on config
     * 
     * @return string
     */
    protected function getServiceUrl()
    {
        return empty($this->viaebayConfig['is_sandbox']) ? self::SERVICE_URL__PROD : self::SERVICE_URL__DEV;
    }

    /**
     * return live or sandbox auth-url depending on config
     *
     * @return string 
     */
    protected function getAuthUrl()
    {
        return empty($this->viaebayConfig['is_sandbox']) ? self::AUTH_URL__PROD : self::AUTH_URL__DEV;
    }

    /**
     * construct
     * 
     * @param oxConfig $oxConfig
     * @param oxSession $oxSession
     * @param array $viaebayConfig
     * @param mo_viaebay__logger $logger 
     */
    public function __construct($oxConfig, $oxSession, $viaebayConfig, mo_viaebay__logger $logger)
    {
        $this->oxConfig = $oxConfig;
        $this->oxSession = $oxSession;
        $this->viaebayConfig = $viaebayConfig;
        $this->logger = $logger;
    }

    /**
     * build params to request session cookie
     *
     * @return array 
     */
    public function buildGetSessionCookie()
    {
        $params = array();
        $params['verb'] = 'POST';
        $params['request_format'] = mo_viaebay__main::$REQUEST_FORMAT__JSON;
        $params['url'] = $this->getAuthUrl() . 'Login';

        //add vendor & version data
        $params['auth_params']['Vendor'] = 'OXID ' . $this->oxConfig->getFullEdition() . ' ' . $this->oxConfig->getVersion();
        $module = oxNew('oxmodule');
        $module->load('mo_viaebay');
        $params['auth_params']['Version'] = $module->getInfo('version');

        $requestParams['userName'] = $this->viaebayConfig['api_username'];
        $requestParams['password'] = $this->viaebayConfig['api_password'];
        $requestParams['createPersistentCookie'] = false;
        $params['request_params'] = $requestParams;

        return $params;
    }

    /**
     * build params to add category
     *
     * @param oxCategory $category
     * @return array 
     */
    public function buildAddCategory(oxCategory $category)
    {
        $params = array();
        $params['verb'] = 'POST';
        $params['request_format'] = mo_viaebay__main::$REQUEST_FORMAT__JSON;
        $params['url'] = $this->getServiceUrl() . 'Catalogs';
        $params['auth_params'] = $this->getAuthParameters();

        $params['request_params'] = $this->getCategoryMasterDataParams($category);

        return $params;
    }

    /**
     * build params to update category
     *
     * @param oxCategory $category
     * @return array 
     */
    public function buildUpdateCategory(oxCategory $category)
    {
        $params = array();
        $params['verb'] = 'PUT';
        $params['request_format'] = mo_viaebay__main::$REQUEST_FORMAT__JSON;
        $params['url'] = $this->getServiceUrl() . 'Catalogs(' . $category->oxcategories__mo_viaebay__via_id->value . 'L)';
        $params['auth_params'] = $this->getAuthParameters();

        $params['request_params'] = $this->getCategoryMasterDataParams($category);

        return $params;
    }

    /**
     * build params to get category master data
     *
     * @param oxCategory $category
     * @return array 
     */
    protected function getCategoryMasterDataParams(oxCategory $category)
    {
        $params = array();
        $params['ForeignId'] = $category->oxcategories__oxid->value;
        $params['Name'] = $category->oxcategories__oxtitle->rawValue;
        $params['IsRootLevel'] = !$category->mo_viaebay__hasParentCategory();
        return $params;
    }

    /**
     * build params to delete category
     *
     * @param string $categoryId
     * @return array 
     */
    public function buildDeleteCategory($categoryId)
    {
        $params = array();
        $params['verb'] = 'DELETE';
        $params['request_format'] = mo_viaebay__main::$REQUEST_FORMAT__JSON;
        $params['url'] = $this->getServiceUrl() . 'Catalogs(' . $categoryId . 'L)';
        $params['auth_params'] = $this->getAuthParameters();

        return $params;
    }

    /**
     * build params to get categories
     *
     * @param array $ids
     * @return array 
     */
    public function buildGetCategories($ids = array())
    {
        $params = array();
        $params['verb'] = 'GET';
        $params['request_format'] = mo_viaebay__main::$REQUEST_FORMAT__JSON;
        $params['url'] = $this->getServiceUrl() . 'Catalogs()?$expand=ChildCatalogs,ParentCatalogs&$select=*,ChildCatalogs,ParentCatalogs';

        if (!empty($ids)) {
            $params['url'] .= '&$filter=(ForeignId+eq+\'';
            $params['url'] .= implode('\')+or+(ForeignId+eq+\'', $ids) . '\')';
        }

        $params['auth_params'] = $this->getAuthParameters();

        return $params;
    }

    /**
     * build params to get category relations
     *
     * @param string $categoryViaId
     * @return array 
     */
    public function buildGetCategoryRelations($categoryViaId)
    {
        $params = array();
        $params['verb'] = 'GET';
        $params['request_format'] = mo_viaebay__main::$REQUEST_FORMAT__JSON;
        $params['url'] = $this->getServiceUrl() . 'Catalogs(' . $categoryViaId . 'L)/ChildCatalogs';
        $params['auth_params'] = $this->getAuthParameters();

        return $params;
    }

    /**
     * build params to set category relation
     *
     * @param string $parentCategoryViaId
     * @param string $subCategoryViaId
     * @return array 
     */
    public function buildSetCategoryRelation($parentCategoryViaId, $subCategoryViaId)
    {
        $params = array();
        $params['verb'] = 'POST';
        $params['request_format'] = mo_viaebay__main::$REQUEST_FORMAT__JSON;
        $params['url'] = $this->getServiceUrl() . 'Catalogs(' . $parentCategoryViaId . 'L)/$links/ChildCatalogs';
        $params['auth_params'] = $this->getAuthParameters();

        $requestParams['uri'] = $this->getServiceUrl() . 'Catalogs(' . $subCategoryViaId . 'L)';
        $params['request_params'] = $requestParams;

        return $params;
    }

    /**
     * build params to delete category relation
     *
     * @param string $parentCategoryViaId
     * @param string $subCategoryViaId
     * @return array 
     */
    public function buildDeleteCategoryRelation($parentCategoryViaId, $subCategoryViaId)
    {
        $params = array();
        $params['verb'] = 'DELETE';
        $params['request_format'] = mo_viaebay__main::$REQUEST_FORMAT__JSON;
        $params['url'] = $this->getServiceUrl() . 'Catalogs(' . $parentCategoryViaId . 'L)/$links/ChildCatalogs('
                . $subCategoryViaId . 'L)';
        $params['auth_params'] = $this->getAuthParameters();

        return $params;
    }

    /**
     * build params to get articles
     *
     * @param array $ids
     * @return array 
     */
    public function buildGetArticles($ids = array(), $options = array())
    {
        $params = array();
        $params['verb'] = 'GET';
        $params['request_format'] = mo_viaebay__main::$REQUEST_FORMAT__JSON;
        $params['url'] = $this->getServiceUrl() . 'Products()?' .
                '$expand=ProductImages,ChildProducts,ParentProducts,ProductSpecifics,Catalogs,ProductVariations,OptionalProductAttributes,ProductVariations/ProductVariationSpecifics&' .
                '$select=*,ProductImages,ChildProducts,ParentProducts,ProductSpecifics,Catalogs,ProductVariations,OptionalProductAttributes,ProductVariations/ProductVariationSpecifics';

        if (!empty($ids) && empty($options[mo_viaebay__feed_articles::OPTION_LOAD_BY_VIA_ID])) {
            $params['url'] .= '&$filter=(ForeignId+eq+\'';
            $params['url'] .= implode('\')+or+(ForeignId+eq+\'', $ids) . '\')';
        } elseif (!empty($ids)) {
            $params['url'] .= '&$filter=(Id+eq+';
            $params['url'] .= implode(')+or+(Id+eq+', $ids) . ')';
        }

        $params['auth_params'] = $this->getAuthParameters();
        return $params;
    }

    /**
     * build params to get article ids
     *
     * @return array 
     */
    public function buildGetArticleIds()
    {
        $params = array();
        $params['verb'] = 'GET';
        $params['request_format'] = mo_viaebay__main::$REQUEST_FORMAT__JSON;
        $params['url'] = $this->getServiceUrl() . 'Products()?$select=Id,ForeignId';
        #$params['url'] .= '&$filter=StockAmount+gt+0';  //doesn't work -> this filter also excludes parent products
        $params['auth_params'] = $this->getAuthParameters();
        return $params;
    }

    /**
     * build params to add article
     *
     * @param oxArticle $article
     * @return array 
     */
    public function buildAddArticle(oxArticle $article)
    {
        $params = array();
        $params['verb'] = 'POST';
        $params['request_format'] = mo_viaebay__main::$REQUEST_FORMAT__JSON;
        $params['url'] = $this->getServiceUrl() . 'Products';
        $params['auth_params'] = $this->getAuthParameters();
        $params['request_params'] = $this->getArticleMasterDataParams($article);
        return $params;
    }

    /**
     * build params to get article master  data
     *
     * @param oxArticle $article
     * @return array 
     */
    protected function getArticleMasterDataParams(oxArticle $article)
    {
        $params = array();
        $params['ForeignId'] = $article->oxarticles__oxid->value;
        $params['Title'] = $article->oxarticles__oxtitle->rawValue;
        $params['Description'] = $article->getLongDescription(true)->rawValue ? $article->getLongDescription(true)->rawValue : '-';
        $params['ShortDescription'] = $article->oxarticles__oxshortdesc->rawValue ? $article->oxarticles__oxshortdesc->rawValue : '-';
        $params['Price'] = $this->getParamPrice($article);
        $params['StockAmount'] = $article->mo_viaebay__getStock();
        $params['Ean'] = $article->oxarticles__oxean->rawValue;
        $params['ProductType'] = $article->mo_viaebay__getProductType();
        $params['Mpn'] = $article->oxarticles__oxmpn->value;
        $params['Brand'] = $article->mo_viaebay__getManufacturername();

        if ($article->oxarticles__oxunitname->value && $article->oxarticles__oxunitquantity->value) {
            $params['UnitQuantity'] = $article->oxarticles__oxunitquantity->value;
            $params['UnitType'] = oxRegistry::get('oxLang')->translateString($article->oxarticles__oxunitname->value, null, false);
        } else {
            $params['UnitQuantity'] = null;
            $params['UnitType'] = null;
        }

        return $params;
    }

    /**
     * build params to get article image data
     *
     * @param oxArticle $article
     * @return array 
     */
    public function getArticleImageDataParams(oxArticle $article)
    {

        $images = array();

        //thumbnail = listing
        $image = array();
        $image['ImageUrl'] = $this->getMasterPicUrl($article->getThumbnailUrl());
        $image['Type'] = mo_viaebay__main::MO_VIAEBAY__IMAGE_TYPE__LISTING;
        $image['ProductId'] = $article->oxarticles__mo_viaebay__via_id->value;
        $images[] = $image;


        $localPics = $article->getPictureGallery();
        foreach ($localPics['Pics'] as $key => $picPath) {
            //set 1st oxpic (key === 1) as Standardpic (2), others = key + 1
            $type = $key === 1 ? mo_viaebay__main::MO_VIAEBAY__IMAGE_TYPE__STANDARD : $key + 1;
            $image = array();
            $image['ImageUrl'] = $this->getMasterPicUrl($picPath);
            $image['Type'] = $type;
            $image['ProductId'] = $article->oxarticles__mo_viaebay__via_id->value;
            $images[] = $image;
        }

        return $images;
    }

    /**
     * generate master URL from generated pic URL
     * @param string $path
     * @return string
     */
    protected function getMasterPicUrl($path)
    {
        $path = str_replace('/generated/', '/master/', $path);
        return preg_replace('#/[0-9]+_[0-9]+_[0-9]+(/[^/]+$)#', '$1', $path);
    }

    /**
     * build params to delete article
     * try to load oxarticle by viaID and update with stock = 0 
     * if article has variants, get variant feed and update variants' stock
     * otherwise delete 
     *
     * @param string $viaID
     * @return array 
     */
    public function buildDeleteArticle($viaID)
    {
        $params = array();
        $oxArticle = oxNew('oxarticle');
        if (!$oxArticle->mo_viaebay__loadByViaId($viaID)) {
            $params['verb'] = 'DELETE';
            $params['request_format'] = mo_viaebay__main::$REQUEST_FORMAT__JSON;
            $params['url'] = $this->getServiceUrl() . 'Products(' . $viaID . 'L)';
            $params['auth_params'] = $this->getAuthParameters();
            return $params;
        } 
        
        if ($oxArticle->getVariantsCount()) {
            //get variant feed from via and set variants' stock = 0
            $feed = mo_viaebay__main::getInstance()->getFeed('articles', array($viaID), array(mo_viaebay__feed_articles::OPTION_LOAD_BY_VIA_ID => true));
            $feedData = $feed->getRecord($viaID);
            foreach ($feedData['ProductVariations'] as $viaVariant) {
                mo_viaebay__main::getInstance()->getInterface()->deleteArticleVariant($viaVariant);
            }
        }
        
        $params = $this->buildUpdateArticle($oxArticle);
        $params['request_params']['StockAmount'] = 0;
        return $params;
    }

    /**
     * build params toupdate article
     *
     * @param oxArticle $article
     * @return array 
     */
    public function buildUpdateArticle(oxArticle $article)
    {
        $params = array();
        $params['verb'] = 'PUT';
        $params['request_format'] = mo_viaebay__main::$REQUEST_FORMAT__JSON;
        $params['url'] = $this->getServiceUrl() . 'Products(' . $article->oxarticles__mo_viaebay__via_id . 'L)';
        $params['auth_params'] = $this->getAuthParameters();
        $params['request_params'] = $this->getArticleMasterDataParams($article);
        return $params;
    }

    /**
     * build params to add article image
     * 
     * @param array $imageData - previously built by getArticleImageDataParams
     * @return array 
     */
    public function buildAddArticleImage($imageData)
    {
        $params = array();
        $params['verb'] = 'POST';
        $params['request_format'] = mo_viaebay__main::$REQUEST_FORMAT__JSON;
        $params['url'] = $this->getServiceUrl() . 'ProductImages';
        $params['auth_params'] = $this->getAuthParameters();
        $params['request_params'] = $imageData;
        return $params;
    }

    /**
     * build params to delete article image
     *
     * @param string $viaId
     * @return array 
     */
    public function buildDeleteArticleImage($viaId)
    {
        $params = array();
        $params['verb'] = 'DELETE';
        $params['request_format'] = mo_viaebay__main::$REQUEST_FORMAT__JSON;
        $params['url'] = $this->getServiceUrl() . 'ProductImages(' . $viaId . 'L)';
        $params['auth_params'] = $this->getAuthParameters();
        return $params;
    }

    /**
     * build params to add article category relatiob
     * 
     * @param string $viaArticleId
     * @param string $viaCategoryId
     * @return array 
     */
    public function buildAddArticleCategoryRelation($viaArticleId, $viaCategoryId)
    {
        $params = array();
        $params['verb'] = 'POST';
        $params['request_format'] = mo_viaebay__main::$REQUEST_FORMAT__JSON;
        $params['url'] = $this->getServiceUrl() . 'Products(' . $viaArticleId . 'L)/$links/Catalogs';
        $params['request_params'] = array('uri' => $this->getServiceUrl() . 'Catalogs(' . $viaCategoryId . 'L)');
        $params['auth_params'] = $this->getAuthParameters();
        return $params;
    }

    /**
     * build params to to delete article category relation
     *
     * @param string $viaArticleId
     * @param string $viaCategoryId
     * @return array 
     */
    public function buildDeleteArticleCategoryRelation($viaArticleId, $viaCategoryId)
    {
        $params = array();
        $params['verb'] = 'DELETE';
        $params['request_format'] = mo_viaebay__main::$REQUEST_FORMAT__JSON;
        $params['url'] = $this->getServiceUrl() . 'Catalogs(' . $viaCategoryId . 'L)/$links/Products(' . $viaArticleId . 'L)';
        $params['auth_params'] = $this->getAuthParameters();
        return $params;
    }

    /**
     * build params to add article attribute
     *
     * @param string $viaArticleId
     * @param string $attributeName
     * @param string $attributeValue
     * @return array 
     */
    public function buildAddArticleAttribute($viaArticleId, $attributeName, $attributeValue)
    {
        $params = array();
        $params['verb'] = 'POST';
        $params['request_format'] = mo_viaebay__main::$REQUEST_FORMAT__JSON;
        $params['url'] = $this->getServiceUrl() . 'ProductSpecifics';
        $params['request_params'] = $this->getArticleAttributeDataParams($viaArticleId, $attributeName, $attributeValue);
        $params['auth_params'] = $this->getAuthParameters();
        return $params;
    }

    /**
     * build params to update article attribute
     *
     * @param string $viaAttributeId
     * @param string $viaArticleId
     * @param string $attributeName
     * @param string $attributeValue
     * @return array 
     */
    public function buildUpdateArticleAttribute($viaAttributeId, $viaArticleId, $attributeName, $attributeValue)
    {
        $params = $this->buildAddArticleAttribute($viaArticleId, $attributeName, $attributeValue);
        $params['verb'] = 'PUT';
        $params['url'] = $this->getServiceUrl() . 'ProductSpecifics(' . $viaAttributeId . 'L)';
        return $params;
    }

        /**
     * build params to add additional information
     *
     * @param string $viaArticleId
     * @param string $attributeName
     * @param string $attributeValue
     * @return array 
     */
    public function buildAddAdditionalInformation($viaArticleId, $attributeName, $attributeValue)
    {
        $params = array();
        $params['verb'] = 'POST';
        $params['request_format'] = mo_viaebay__main::$REQUEST_FORMAT__JSON;
        $params['url'] = $this->getServiceUrl() . 'OptionalProductAttributes';
        //same data as article attributes
        $params['request_params'] = $this->getArticleAttributeDataParams($viaArticleId, $attributeName, $attributeValue);
        $params['auth_params'] = $this->getAuthParameters();
        return $params;
    }
    
     /**
     * build params to update additional information
     *
     * @param string $viaAttributeId
     * @param string $viaArticleId
     * @param string $attributeName
     * @param string $attributeValue
     * @return array 
     */
    public function buildUpdateAdditionalInformation($viaAttributeId, $viaArticleId, $attributeName, $attributeValue) 
    {
        $attributeValue = "Leder";
        $params = $this->buildAddAdditionalInformation($viaArticleId, $attributeName, $attributeValue);
        $params['verb'] = 'PUT';
        $params['url'] = $this->getServiceUrl() . 'OptionalProductAttributes(' . $viaAttributeId . 'L)';
        return $params;
    }
    
    /**
     * build params to update additional information
     *
     * @param string $viaAttributeId
     * @param string $viaArticleId
     * @param string $attributeName
     * @param string $attributeValue
     * @return array 
     */
    public function buildDeleteAdditionalInformation($viaAttributeId) 
    {
        $params = array();
        $params['verb'] = 'DELETE';
        $params['request_format'] = mo_viaebay__main::$REQUEST_FORMAT__JSON;
        $params['url'] = $this->getServiceUrl() . 'OptionalProductAttributes(' . $viaAttributeId . 'L)';
        $params['auth_params'] = $this->getAuthParameters();
        return $params;
    }
    
    
    /**
     * build params to ger article attribute data
     *
     * @param string $viaArticleId
     * @param string $attributeName
     * @param string $attributeValue
     * @return array 
     */
    public function getArticleAttributeDataParams($viaArticleId, $attributeName, $attributeValue)
    {
        $attributeData = array();
        $attributeData['ProductId'] = $viaArticleId;
        $attributeData['Name'] = $attributeName;
        $attributeData['Value'] = $attributeValue;

        return $attributeData;
    }

    /**
     * build params to delete article attribute
     *
     * @param string $viaArticleAttributeId
     * @return array 
     */
    public function buildDeleteArticleAttribute($viaArticleAttributeId)
    {
        $params = array();
        $params['verb'] = 'DELETE';
        $params['request_format'] = mo_viaebay__main::$REQUEST_FORMAT__JSON;
        $params['url'] = $this->getServiceUrl() . 'ProductSpecifics(' . $viaArticleAttributeId . 'L)';
        $params['auth_params'] = $this->getAuthParameters();
        return $params;
    }

    /**
     * build params to get orders
     *
     * @param array $ids
     * @return array 
     */
    public function buildGetOrders($ids = array())
    {
        $params = array();
        $params['verb'] = 'GET';
        $params['request_format'] = mo_viaebay__main::$REQUEST_FORMAT__JSON;
        $params['url'] = $this->getServiceUrl() . 'SalesOrders()?' .
                '$expand=Address,ShippingAddress,Buyer,SalesOrderItems,SalesOrderItems/ProductVariation&' .
                '$select=*,Address,ShippingAddress,Buyer,SalesOrderItems,SalesOrderItems/ProductVariation';

        if (!empty($ids)) {
            $params['url'] .= '+and+(ForeignOrderId+eq+\'';
            $params['url'] .= implode('\')+or+(ForeignOrderId+eq+\'', $ids) . '\')';
        } else {
            $params['url'] .= '&$filter=((ForeignOrderId+eq+null)+or+(ForeignOrderId+eq+\'\'))';
        }


        $params['auth_params'] = $this->getAuthParameters();

        return $params;
    }

    /**
     * update Order - allowed update fields: PaymentStatus, ForeignOrderId, PaidAmount, PaidDate
     * 
     * => only update one property at once
     * 
     * @param string $viaId
     * @param string $propertyName
     * @param string $propertyValue
     * @return array 
     */
    public function buildUpdateOrder($viaId, $propertyName, $propertyValue)
    {
        $params = array();
        $params['verb'] = 'PUT';
        $params['request_format'] = mo_viaebay__main::$REQUEST_FORMAT__JSON;
        $params['url'] = $this->getServiceUrl() . 'SalesOrders(' . $viaId . 'L)/' . $propertyName;
        $params['auth_params'] = $this->getAuthParameters();
        $params['request_params'] = array($propertyName => $propertyValue);
        return $params;
    }

    /**
     * build params to report payment
     *
     * @param oxOrder $order
     * @return array 
     */
    public function buildReportPayment(oxOrder $order)
    {
        return $this->buildUpdateOrder($order->oxorder__mo_viaebay__via_id->value, 'PaidDate', $order->oxorder__oxpaid->value);
    }

    /**
     * build params to report shipment
     *
     * @param oxOrder $order
     * @return array 
     */
    public function buildReportShipment(oxOrder $order)
    {
        return $this->buildUpdateOrder($order->oxorder__mo_viaebay__via_id->value, 'ShippingStatus', $order->oxorder__mo_viaebay__shipping_state->value);
    }

    /**
     * build params to report trackingcode
     */
    public function buildReportTrackingcode(oxOrderArticle $article, $trackCode)
    {
        return $this->buildUpdateShipment($article->oxorderarticles__mo_viaebay__via_id->value, 'TrackingNumbers' ,$trackCode);
    }
    
   /**
    *  build params to report CarrierUsed
    * 
    * @return array
    */
    
    public function buildReportCarrierUsed(oxOrderArticle $article, $carrier)
    {
        return $this->buildUpdateShipment($article->oxorderarticles__mo_viaebay__via_id->value, 'CarrierUsed' , $carrier);
    }

    
    /**
     * update an article e.g. with TrackingCode or CarrierUsed
     * 
     * @param type $viaAttributeId
     * @param type $propertyName
     * @param type $propertyValue
     * @return type
     */
    public function buildUpdateShipment($viaAttributeId, $propertyName, $propertyValue)
    {
        $params = array();
        $params['verb'] = 'PUT';
        $params['request_format'] = mo_viaebay__main::$REQUEST_FORMAT__JSON;
        $params['url'] = $this->getServiceUrl() . 'SalesOrderItems(' . $viaAttributeId . 'L)/'. $propertyName;
        $params['request_params'] = array($propertyName => $propertyValue);
        $params['auth_params'] = $this->getAuthParameters();
        return $params;
    }
    
    /**
     * build params to add article variant
     *
     * @param string $viaId
     * @param oxArticle $variant
     * @return array 
     */
    public function buildAddArticleVariant($viaId, oxArticle $variant)
    {
        $params = array();
        $params['verb'] = 'POST';
        $params['request_format'] = mo_viaebay__main::$REQUEST_FORMAT__JSON;
        $params['url'] = $this->getServiceUrl() . 'ProductVariations';
        $params['auth_params'] = $this->getAuthParameters();
        $params['request_params'] = array(
            'ProductId' => $viaId,
            'Price' => $this->getParamPrice($variant),
            'StockAmount' => $variant->mo_viaebay__getStock(),
            'Sku' => $variant->getId(),
            'Ean' => $variant->mo_viaebay__getEanForVariant(),
            'Mpn' => $variant->mo_viaebay__getMpnForVariant(),
        );
        return $params;
    }

    /**
     * build params to update article variant
     *
     * @param oxArticle $oxVariant
     * @return array 
     */
    public function buildUpdateArticleVariant(oxArticle $oxVariant)
    {
        $viaId = $oxVariant->oxarticles__mo_viaebay__via_id->value;
        $parentViaId = $oxVariant->getParentArticle()->oxarticles__mo_viaebay__via_id->value;
        $params = $this->buildAddArticleVariant($parentViaId, $oxVariant);
        $params['verb'] = 'PUT';
        $params['url'] = $this->getServiceUrl() . 'ProductVariations(' . $viaId . 'L)';
        return $params;
    }

    /**
     * update article variant to stockamount = 0
     *
     * @param string $variantId, $productId
     * @return array 
     */
    public function buildDeleteArticleVariant($viaVariant)
    {
        $params = array();
        $params['verb'] = 'PUT';
        $params['request_format'] = mo_viaebay__main::$REQUEST_FORMAT__JSON;
        $params['url'] = $this->getServiceUrl() . 'ProductVariations(' . $viaVariant['Id'] . 'L)';
        $params['auth_params'] = $this->getAuthParameters();
        $params['request_params'] = array(
            'ProductId' => $viaVariant['ProductId'],
            'Price' => $viaVariant['Price'],
            'StockAmount' => 0,
            'Sku' => $viaVariant['Sku'],
            'Ean' => $viaVariant['Ean'],
            'Mpn' => $viaVariant['Mpn'],
        );

        return $params;
    }

    /**
     * build params to add article variant specifics
     *
     * @param string $viaId
     * @param string $name
     * @param string $value
     * @return array 
     */
    public function buildAddArticleVariantSpecific($viaId, $name, $value)
    {
        $params = array();
        $params['verb'] = 'POST';
        $params['request_format'] = mo_viaebay__main::$REQUEST_FORMAT__JSON;
        $params['url'] = $this->getServiceUrl() . 'ProductVariationSpecifics';
        $params['auth_params'] = $this->getAuthParameters();
        $params['request_params'] = array(
            'ProductVariationId' => $viaId,
            'Name' => $name,
            'Value' => $value,
        );
        return $params;
    }

    /**
     * build params to delete article variant specifics
     *
     * @param string $viaId
     * @return array 
     */
    public function buildDeleteArticleVariantSpecific($viaId)
    {
        $params = array();
        $params['verb'] = 'DELETE';
        $params['request_format'] = mo_viaebay__main::$REQUEST_FORMAT__JSON;
        $params['url'] = $this->getServiceUrl() . 'ProductVariationSpecifics(' . $viaId . 'L)';
        $params['auth_params'] = $this->getAuthParameters();
        return $params;
    }

    public function buildSetProductProfile($productId, $profileId, $type)
    {
        $params = array();
        $params['verb'] = 'POST';
        $params['request_format'] = mo_viaebay__main::$REQUEST_FORMAT__JSON;
        $params['url'] = $this->getServiceUrl() . 'SetProductProfile?productId=' . $productId . 'L&type=' . $type .
                '&profileId=' . $profileId . 'L';
        $params['auth_params'] = $this->getAuthParameters();
        return $params;
    }

    /**
     * returns auth-parameters for API-calls
     * 
     * @return string 
     */
    protected function getAuthParameters()
    {
        $authParameters = array();
        $authParameters['SubscriptionToken'] = $this->viaebayConfig['subscription_token'];
        $authParameters['Cookie'] = $this->oxSession->getVariable('mo_viaebay__session_cookie');
        
        return $authParameters;
    }

    /**
     * update auth params (session cookie)
     * 
     * @param array $params
     * @return array 
     */
    public function updateAuthParameters($params)
    {
        $params['auth_params'] = $this->getAuthParameters();
        return $params;
    }

    protected function getParamPrice(oxArticle $oxarticle)
    {
        return number_format($oxarticle->getPrice()->getBruttoPrice(), 2, '.', '');
    }

    /**
     * update auth params (session cookie)
     * 
     * @param oxArticle $oxArticle
     * @return array 
     */
    public function buildUpdateInventory(oxArticle $oxArticle)
    {
        $params = array();
        $params['verb'] = 'POST';
        $params['request_format'] = mo_viaebay__main::$REQUEST_FORMAT__JSON;

        $urlParams = array(
            'stockAmount' => $oxArticle->mo_viaebay__getStock(),
            'price' => $this->getParamPrice($oxArticle) . 'm',
        );

        if ($oxArticle->isVariant()) {
            $urlParams['productId'] = $oxArticle->getParentArticle()->oxarticles__mo_viaebay__via_id->value . 'L';
            $urlParams['productVariationId'] = $oxArticle->oxarticles__mo_viaebay__via_id->value . 'l';
        } else {
            $urlParams['productId'] = $oxArticle->oxarticles__mo_viaebay__via_id->value . 'L';
        }

        $params['url'] = $this->getServiceUrl() . 'ReviseInventoryStatus?' . http_build_query($urlParams);
        $params['auth_params'] = $this->getAuthParameters();

        return $params;
    }

}
