<?php

/**
 * mo_viaebay__sync_manager_articles
 * 
 * manage article sync with via
 * 
 * @version   $Id: mo_viaebay__sync_manager_articles.php 219 2013-06-06 13:07:53Z $
 */

/**
 * manage article sync with via
 * 
 * @package   mo_viaebay
 * @extend mo_viaebay__sync_manager 
 */
class mo_viaebay__sync_manager_articles extends mo_viaebay__sync_manager
{

    /**
     * @staticvar string
     */
    const PRODUCT_SPECIFIC_VALUE__MISC = 'verschiedene';
    //deleted diverse 
    const PRODUCT_SPECIFIC_LABEL__MISC = '';

    /**
     * store local images
     * 
     * @var array 
     */
    protected $localImages = array();
    protected $oldLongDescription = '';
    
    // saves the lowest Price of an article and his variants for the delivery calculation
    protected $lowestPrice = 0;

    /**
     * sync article
     * 
     * @param oxBase $article 
     */
    public function syncObject(oxBase $article)
    {
        $this->lowestPrice = 0;
        $this->setPriceGroup();
        try {
            parent::syncObject($article);
            //set long description
            $this->oldLongDescription = $this->syncObject->getLongDescription()->rawValue;
            $this->syncObject->setArticleLongDesc($this->getParamLongDescription());
            $this->syncObject->load($this->syncObject->getId());
            $this->syncObjectViaId = $this->syncObject->oxarticles__mo_viaebay__via_id->value;
            //need objectFeedData for checking if the viaID exist
            $this->objectFeedData = $this->feed->getRecord($this->syncObjectViaId);
            if ($this->syncObject->getPrice()->getBruttoPrice()) {
                $this->lowestPrice = $this->syncObject->getPrice()->getBruttoPrice();
            }
            if (!$this->syncObjectViaId || !$this->objectFeedData) {
                $this->add();
            } else {
                $this->syncArticle();
            }

            $this->syncedObjectIds[] = $this->syncObjectViaId;
            $this->syncArticleImages();
            $this->syncArticleCategoryRelations();
            $this->syncArticleAttributes();
            $this->syncArticleVariants();
            $this->setProfile();

            //reset long description
            $this->syncObject->setArticleLongDesc($this->oldLongDescription);

            $this->markObjectAsSynced();
        } catch (oxException $oEx) {
            //on error, remove article sync flag
            $article->oxarticles__mo_viaebay__allow_export = new oxField(0);
            $article->save();
            $this->logger->error($oEx->getMessage());
            return false;
        }
        return true;
    }

    /**
     * sends sync object to via and sets via id 
     * 
     * @return boolean 
     */
    protected function add()
    {
        $this->logger->logExecution();
        if (!$this->syncObjectViaId = mo_viaebay__main::getInstance()->getInterface()->addArticle($this->syncObject)) {
            return false;
        }
        $this->syncObject->oxarticles__mo_viaebay__via_id = new oxField($this->syncObjectViaId);
        return $this->syncObject->save();
    }

    /**
     * sync article ** won't be invoked anymore
     */
    protected function syncArticle()
    {
//        $this->compareProperty($this->objectFeedData['ForeignId'], $this->syncObject->oxarticles__oxid->value, 'ForeignId');
//        $this->compareProperty($this->objectFeedData['Title'], $this->syncObject->oxarticles__oxtitle->rawValue, 'Title');
//        $this->compareProperty($this->objectFeedData['Description'], $this->syncObject->getLongDescription(true)->rawValue, 'Description', true);
//        $this->compareProperty($this->objectFeedData['ShortDescription'], $this->syncObject->oxarticles__oxshortdesc->rawValue, 'ShortDescription', true);
//        $this->compareProperty($this->objectFeedData['Price'], $this->syncObject->getPrice()->getBruttoPrice(), 'Price');
//        $this->compareProperty($this->objectFeedData['StockAmount'], $this->syncObject->mo_viaebay__getStock(), 'StockAmount');
//        $this->compareProperty($this->objectFeedData['ProductType'], $this->syncObject->mo_viaebay__getProductType(), 'ProductType');
//        $this->compareProperty((float) $this->objectFeedData['UnitQuantity'], $this->syncObject->oxarticles__oxunitquantity->value, 'UnitQuantity');
//        $this->compareProperty($this->objectFeedData['UnitType'], oxRegistry::get('oxLang')->translateString($this->syncObject->oxarticles__oxunitname->value, null, false), 'UnitType');
//        $this->compareProperty($this->objectFeedData['Ean'], $this->syncObject->oxarticles__oxean->rawValue, 'Ean');
//        $this->compareProperty($this->objectFeedData['Mpn'], $this->syncObject->oxarticles__oxmpn->value, 'Mpn');
//        $this->compareProperty($this->objectFeedData['Brand'], $this->syncObject->mo_viaebay__getManufacturername(), 'Brand');
        
        if ($this->currentObjectNeedsUpdate || true) {
            mo_viaebay__main::getInstance()->getInterface()->updateArticle($this->syncObject);
        }
    }

    protected function getParamLongDescription()
    {
        $attributeDescription = "";
        $description = $this->syncObject->getLongDescription()->rawValue;

        if (!$attributes = $this->fetchLocalAttributes()) {
            return $description;
        }


        foreach ($attributes as $name => $value) {
            $viaAttribute = $this->getViaAttributesByTitle($name);
            if ($attributeName = $this->getViaAttributeName($name)) {
                $name = $attributeName;
            }
            // if Attribute should transmitted as description do ...
            if (($viaAttribute == 3) || (($viaAttribute == 0) && ($this->viaConfig['attributes_as'] == 2))) {
                $attributeDescription .= "<tr><td>" . $name . "</td><td>" . $value . "</td></tr>\n";
            }
        }
        if ($attributeDescription) {
            $description .= "\n<table>\n";
            $description .= $attributeDescription;
            $description .= "</table>";
        }
        return $description;
    }

    /**
     * compare property
     * 
     * @param string $viaProperty
     * @param string $localValue 
     * @param string $name 
     */
    protected function compareProperty($viaProperty, $localValue, $name, $hasPlaceholder = false)
    {
        if ($viaProperty != $localValue) {
            //placeholder ?
            if (!$localValue && ($viaProperty == '-') && $hasPlaceholder) {
                return;
            }

            $diff = $this->generateDiff($viaProperty, $localValue);
            $this->logger->info('syncObject mismatch (' . $name . "): \n" . $diff);
            $this->currentObjectNeedsUpdate = true;
        }
    }

    protected function generateDiff($string1, $string2)
    {
        $diff = array();
        $string1 = explode("\n", $string1);
        $string2 = explode("\n", $string2);

        foreach ($string1 as $lineNumber1 => $line1) {
            $matchLine = array_search($line1, $string2);

            if ($matchLine === false) {
                $diff[$lineNumber1][0] = '- ' . $line1;
            } elseif ($matchLine !== $lineNumber1) {
                $diff[$lineNumber1][0] = '- ' . $line1;
                $diff[$matchLine][1] = '+ ' . $line1;
                unset($string2[$matchLine]);
            } else {
                unset($string2[$lineNumber1]);
            }
            unset($string1[$lineNumber1]);
        }

        foreach ($string2 as $lineNumber2 => $line2) {
            $diff[$lineNumber2][1] = '+ ' . $line2;
        }

        foreach ($diff as $number => $values) {
            foreach ($values as $value) {
                $output .= $number . ' ' . $value . "\n";
            }
        }
        return $output;
    }

    /**
     * sync article images
     */
    protected function syncArticleImages()
    {
        $this->localImages = $this->paramBuilder->getArticleImageDataParams($this->syncObject);

//delete async parent assignments
        $this->deleteAsyncArticleImages();

//add
        foreach ($this->localImages as $image) {
            if (empty($this->objectFeedData['ProductImages']) || !in_array(array('ImageUrl' => $image['ImageUrl']), $this->objectFeedData['ProductImages'])) {
                mo_viaebay__main::getInstance()->getInterface()->addArticleImage($image);
            }
        }
    }

    /**
     * delete article images
     * 
     * @return mixed 
     */
    protected function deleteAsyncArticleImages()
    {
        if (empty($this->objectFeedData['ProductImages'])) {
            return;
        }

        $viaImages = $this->objectFeedData['ProductImages'];
        $this->removeArticleImages($viaImages);
    }

    /**
     * remove article images
     * 
     * @param array $viaImages
     * @return void 
     */
    protected function removeArticleImages($viaImages)
    {
        foreach ($viaImages as $viaImage) {
            if (($key = $this->arraySearchAssoc($viaImage['ImageUrl'], $this->localImages, 'ImageUrl')) === false) {
                mo_viaebay__main::getInstance()->getInterface()->deleteArticleImage($viaImage['Id']);
                continue;
            }

            unset($this->localImages[$key]);
        }
    }

    /**
     * sync article - category relations
     */
    protected function syncArticleCategoryRelations()
    {
        $this->localCatRelations = $this->syncObject->mo_viaebay__getViaCategoryIds();
//delete async parent assignments
        $this->deleteAsyncCategoryRelations();
//add
        $this->addCategoryRelations();
    }

    /**
     * delete category relations
     *
     * @return void 
     */
    protected function deleteAsyncCategoryRelations()
    {
        if (empty($this->objectFeedData['Catalogs'])) {
            return;
        }
        $this->removeCategoryRelations();
    }

    /**
     * remove category relations
     */
    protected function removeCategoryRelations()
    {
        foreach ($this->objectFeedData['Catalogs'] as $viaCat) {
            if (($key = array_search($viaCat['Id'], $this->localCatRelations)) === false) {
                mo_viaebay__main::getInstance()->getInterface()->deleteArticleCategoryRelation(
                        $this->syncObject->oxarticles__mo_viaebay__via_id->value, $viaCat['Id']);
            } else {
                unset($this->localCatRelations[$key]);
            }
        }
    }

    protected function setProfile()
    {
//check if profile handling is necessary
        if (!$this->viaConfig['free_delivery__profile_id'] || !$this->viaConfig['standard_delivery__profile_id']) {
            return;
        }

        mo_viaebay__main::getInstance()->getInterface()->setProductProfile(
                $this->syncObject->oxarticles__mo_viaebay__via_id->value, $this->getShippingProfileId(), mo_viaebay__main::MO_VIAEBAY__PROFILE_ID__SHIPPING);
    }

    /**
     * check for other delivery profiles (free delivery)
     * @return boolean
     */
    protected function getShippingProfileId()
    {
        if ($this->lowestPrice && $this->lowestPrice >= $this->viaConfig['free_delivery__amount']) {
            return $this->viaConfig['free_delivery__profile_id'];
        }
        return $this->viaConfig['standard_delivery__profile_id'];
    }

    /**
     * add category relations
     */
    protected function addCategoryRelations()
    {
        foreach ($this->localCatRelations as $relation) {
            if (empty($this->objectFeedData['Catalogs']) || !in_array($relation, $this->objectFeedData['Catalogs'])) {
                mo_viaebay__main::getInstance()->getInterface()->addArticleCategoryRelation(
                        $this->syncObject->oxarticles__mo_viaebay__via_id->value, $relation);
            }
        }
    }

    /**
     * sync article attributes
     */
    protected function syncArticleAttributes()
    {
        $localAttributes = $this->fetchLocalAttributes();
//trim attributes
        $localAttributes = $this->trimAttributes($localAttributes);
        foreach ($localAttributes as $attributeName => $attributeValue) {
            if ($this->viaConfig['ignore_attributes']) {

//ignore all attributes except Marke
            } else {
                $viaAttribute = $this->getViaAttributesByTitle($attributeName);
                if ($viaName = $this->getViaAttributeName($attributeName)) {
                    $attributeName = $viaName;
                }
                // if Attribute should transmitted as Addtional Information
                if (($viaAttribute == 2) || (($viaAttribute == 0) && ($this->viaConfig['attributes_as'] == 1))) {
                    if (empty($this->objectFeedData['OptionalProductAttributes'][$attributeName])) {
                        mo_viaebay__main::getInstance()->getInterface()->addAdditionalInformation($this->syncObjectViaId, $attributeName, $attributeValue);
                    } else {
                        if ($this->objectFeedData['OptionalProductAttributes'][$attributeName]['Value'] != $attributeValue) {
                            mo_viaebay__main::getInstance()->getInterface()->deleteAdditionalInformation($this->objectFeedData['OptionalProductAttributes'][$attributeName]['Id']);
                            mo_viaebay__main::getInstance()->getInterface()->addAdditionalInformation($this->syncObjectViaId, $attributeName, $attributeValue);
                        }
                        unset($this->objectFeedData['OptionalProductAttributes'][$attributeName]);
                    }
                }
                // if Attribute should transmitted as characteristic
                if (($viaAttribute == 1) || (($viaAttribute == 0) && ($this->viaConfig['attributes_as'] == 0))) {
                    if (empty($this->objectFeedData['ProductSpecifics'][$attributeName])) {
                        mo_viaebay__main::getInstance()->getInterface()->addArticleAttribute($this->syncObjectViaId, $attributeName, $attributeValue);
                    } else {
                        if ($this->objectFeedData['ProductSpecifics'][$attributeName]['Value'] != $attributeValue) {
                            mo_viaebay__main::getInstance()->getInterface()->updateArticleAttribute(
                                    $this->objectFeedData['ProductSpecifics'][$attributeName]['Id'], $this->syncObjectViaId, $attributeName, $attributeValue);
                        }
                        unset($this->objectFeedData['ProductSpecifics'][$attributeName]);
                    }
                }
            }
        }


        //delete remaing attributes in feed-data
        $this->deleteAsyncArticleAttributes();
        $this->deleteAsyncAdditionalInformation();
    }

    /**
     * fetch local attributes and other properties
     *
     * @return array 
     */
    protected function fetchLocalAttributes()
    {
        $attributes = array();
        foreach ($this->syncObject->getAttributes()->getArray() as $oxAttribute) {
            $attributes[$oxAttribute->oxattribute__oxtitle->rawValue] = $oxAttribute->oxattribute__oxvalue->rawValue;
        }
        foreach ($this->getVariantProperties() as $oxVariantSelectList) {
            //append suffix to att-name ebay does not allow double var-properties and attributes
            $name = $oxVariantSelectList->getLabel() . self::PRODUCT_SPECIFIC_LABEL__MISC;
            $attributes[$name] = self::PRODUCT_SPECIFIC_VALUE__MISC;
        }
        return $attributes;
    }

    /**
     * trim attributes according to VIA-interface
     * @param type $attributes
     */
    protected function trimAttributes($attributes)
    {
        foreach ($attributes as $key => $value) {
            if (strlen($key) > 40) {
                unset($attributes[$key]);
                $key = substr($key, 0, 40);
                $attributes[$key] = $value;
            }
            if (strlen($value) > 50) {
                $attributes[$key] = substr($value, 0, 50);
            }
        }
        return $attributes;
    }

    /**
     * get variant properties
     *
     * @return array 
     */
    protected function getVariantProperties()
    {

        if (!$oxVariantSelections = $this->syncObject->getVariantSelections()) {
            return array();
        }

        if (empty($oxVariantSelections['selections'])) {
            return array();
        }

        return $oxVariantSelections['selections'];
    }

    /**
     * sync article variants 
     */
    protected function syncArticleVariants()
    {
        if (!$oxVariantSelections = $this->syncObject->getVariantSelections()) {
            $this->deleteArticleVariants();
            return;
        }
        $variantProperties = $oxVariantSelections['selections'];

        foreach ($oxVariantSelections['rawselections'] as $key => $variantData) {
//load variant
            $oxVariant = oxNew('oxarticle');

            if (!$oxVariant->load($key)) {
                continue;
            }
            
            if (!$oxVariant->oxarticles__mo_viaebay__allow_export->value) {
                continue;
            }
            
//min stock
            if (!$this->checkVariantStock($oxVariant)) {
                continue;
            }
            //set lowest price
            $price = $oxVariant->getPrice()->getBruttoPrice();
            if ($price < $this->lowestPrice || !$this->lowestPrice) {
                $this->lowestPrice = $price;
            }

            if (empty($this->objectFeedData['ProductVariations'][$oxVariant->oxarticles__mo_viaebay__via_id->value])) {
                $this->addVariant($oxVariant);
            } else {
//compare
                $viaVariant = $this->objectFeedData['ProductVariations'][$oxVariant->oxarticles__mo_viaebay__via_id->value];

                $this->compareVariantData($oxVariant, $viaVariant);
            }

            $this->syncVariantSpecifics($oxVariant, $variantData, $variantProperties);

            if (isset($this->objectFeedData['ProductVariations'][$oxVariant->oxarticles__mo_viaebay__via_id->value])) {
                unset($this->objectFeedData['ProductVariations'][$oxVariant->oxarticles__mo_viaebay__via_id->value]);
            }
        }

        $this->deleteArticleVariants();
    }

    /**
     * delete all remaining variants in report
     * @return void 
     */
    protected function deleteArticleVariants()
    {
        if (empty($this->objectFeedData['ProductVariations'])) {
            return;
        }

        foreach ($this->objectFeedData['ProductVariations'] as $viaVariant) {
            mo_viaebay__main::getInstance()->getInterface()->deleteArticleVariant($viaVariant);
        }
    }

    /**
     * compare variant's main data via-data with oxid
     * @param oxArticle $oxVariant
     * @param array $viaVariant
     * @return boolean
     */
    protected function compareVariantData(oxArticle $oxVariant, $viaVariant)
    {
        if ($oxVariant->getPrice()->getBruttoPrice() != $viaVariant['Price']) {
            return mo_viaebay__main::getInstance()->getInterface()->updateArticleVariant($oxVariant);
        }
        if ($oxVariant->mo_viaebay__getStock() != $viaVariant['StockAmount']) {
            return mo_viaebay__main::getInstance()->getInterface()->updateArticleVariant($oxVariant);
        }
        if ($oxVariant->getId() != $viaVariant['Sku']) {
            return mo_viaebay__main::getInstance()->getInterface()->updateArticleVariant($oxVariant);
        }
        if ($oxVariant->mo_viaebay__getEanForVariant() != $viaVariant['Ean']) {
            return mo_viaebay__main::getInstance()->getInterface()->updateArticleVariant($oxVariant);
        }
        if ($oxVariant->mo_viaebay__getMpnForVariant() != $viaVariant['Mpn']) {
            return mo_viaebay__main::getInstance()->getInterface()->updateArticleVariant($oxVariant);
        }
    }

    /**
     * get specifics from feed
     * @param oxArticle $oxVariant
     * @return array 
     */
    protected function getVariantSpecificsFromDataFeed(oxArticle $oxVariant)
    {
        if (!empty($this->objectFeedData['ProductVariations'][$oxVariant->oxarticles__mo_viaebay__via_id->value]['ProductVariationSpecifics'])) {
            return $this->objectFeedData['ProductVariations'][$oxVariant->oxarticles__mo_viaebay__via_id->value]['ProductVariationSpecifics'];
        }

        return array();
    }

    /**
     * sync variant properties
     * 
     * @param oxArticle $oxVariant
     * @param type $variantData
     * @param type $variantProperties 
     */
    protected function syncVariantSpecifics(oxArticle $oxVariant, $variantData, $variantProperties)
    {
//get specifics from feed
        $viaSpecifics = $this->getVariantSpecificsFromDataFeed($oxVariant);

        foreach ($variantData as $key => $data) {
//map data to property
            $propertyName = $variantProperties[$key]->getLabel();
            $propertyValue = $data['name'];

            if (empty($viaSpecifics[$propertyName]['Value']) || $viaSpecifics[$propertyName]['Value'] != $propertyValue) {
                mo_viaebay__main::getInstance()->getInterface()->addArticleVariantSpecific($oxVariant->oxarticles__mo_viaebay__via_id->value, $propertyName, $propertyValue);
            } else {
                unset($viaSpecifics[$propertyName]);
            }
        }

//delete 
        foreach ($viaSpecifics as $spec) {
            mo_viaebay__main::getInstance()->getInterface()->deleteArticleVariantSpecific($spec['Id']);
        }
    }

    /**
     * check if "real" variant stock is larger than filter setting
     * @param oxArticle $variant
     * @return boolean 
     */
    protected function checkVariantStock(oxArticle $variant)
    {
        if (empty($this->viaConfig['mo_viaebay__filter']['min_stock'])) {
            return true;
        }
//contract warehouse
        if ((int) $variant->oxarticles__oxstockflag->value === 4) {
            return true;
        }

        if ($variant->oxarticles__oxstock->value >= $this->viaConfig['mo_viaebay__filter']['min_stock']) {
            return true;
        }

        return false;
    }

    /**
     * add variant - call API and save via-id
     * @param oxArticle $variant 
     */
    protected function addVariant(oxArticle $variant)
    {
        $id = mo_viaebay__main::getInstance()->getInterface()->addArticleVariant($this->syncObjectViaId, $variant);
        $variant->oxarticles__mo_viaebay__via_id = new oxField($id);
        $variant->save();
    }

    /**
     * delete remaining article attributes
     *
     * @return void 
     */
    protected function deleteAsyncArticleAttributes()
    {
        if (empty($this->objectFeedData['ProductSpecifics'])) {
            return;
        }

        foreach ($this->objectFeedData['ProductSpecifics'] as $viaAttribute) {
            mo_viaebay__main::getInstance()->getInterface()->deleteArticleAttribute($viaAttribute['Id']);
        }
    }

    /**
     * delete remaining article attributes
     *
     * @return void 
     */
    protected function deleteAsyncAdditionalInformation()
    {
        if (empty($this->objectFeedData['OptionalProductAttributes'])) {
            return;
        }
        foreach ($this->objectFeedData['OptionalProductAttributes'] as $viaAttribute) {
            mo_viaebay__main::getInstance()->getInterface()->deleteAdditionalInformation($viaAttribute['Id']);
        }
    }

    /**
     * search array
     *
     * @param string $needle
     * @param array $haystack
     * @param string $index
     * @return mixed 
     */
    protected function arraySearchAssoc($needle, $haystack, $index)
    {
        foreach ($haystack as $key => $value) {
            if ($value instanceof oxBase) {
                $compareValue = $value->$index->rawValue;
            } else {
                $compareValue = $value[$index];
            }


            if ($compareValue == $needle) {
                return $key;
            }
        }
        return false;
    }

    /**
     * delete data
     *
     * @param array $oxidArticleViaIds
     * @param mo_viaebay__feed_article_ids $viaArticleIdsFeed 
     */
    public function deleteAsyncData($oxidArticleViaIds, mo_viaebay__feed_article_ids $viaArticleIdsFeed)
    {
        foreach ($viaArticleIdsFeed->getData() as $viaId => $record) {
            if (!in_array($viaId, $oxidArticleViaIds)) {
                $this->logger->logExecution();
                mo_viaebay__main::getInstance()->getInterface()->deleteArticle($viaId);
            }
        }
    }

    /**
     * mark object as synced
     */
    protected function markObjectAsSynced()
    {
        $this->syncObject->oxarticles__mo_viaebay__last_sync = new oxField(date('Y-m-d H:i:s', time()));
        $this->syncObject->save();
    }

    /**
     * get the via configuration for the attributes
     */
    protected function getViaAttributesByTitle($label)
    {
        $db = oxdb::getDb();
        $sql = "SELECT mo_viaebay__attributes_as
            FROM " . getViewName('oxattribute') . " 
            WHERE oxtitle = " . $db->quote($label) . " ";

        return oxDb::getDb()->GetOne($sql);
    }

    protected function getViaAttributeName($label)
    {
        $db = oxdb::getDb();
        $sql = "SELECT mo_viaebay__attribute_name
            FROM " . getViewName('oxattribute') . " 
            WHERE oxtitle = " . $db->quote($label) . " ";

        return oxDb::getDb()->GetOne($sql);
    }

}
