<?php

/**
 * mo_viaebay__sync_manager_categories
 * 
 * manage category sync with via
 * 
 * @version   $Id: mo_viaebay__sync_manager_categories.php 154 2012-12-14 16:00:40Z $
 */

/**
 * manage category sync with via
 * 
 * @package   mo_viaebay
 * @extend mo_viaebay__sync_manager 
 */
class mo_viaebay__sync_manager_categories extends mo_viaebay__sync_manager
{

  /**
   * sync category
   *
   * @param oxBase $category 
   */
  public function syncObject(oxBase $category)
  {
    parent::syncObject($category);

    //reset parent category => force reload with possible new via-id
    $this->syncObject->setParentCategory(null);
    //$viaId = $this->syncObject->oxcategories__mo_viaebay__via_id->value;
    $this->syncObject->load($this->syncObject->getId());
    $viaId = $this->syncObject->oxcategories__mo_viaebay__via_id->value;
    $this->objectFeedData = $this->feed->getRecord($viaId);

    if (!$viaId || !$this->objectFeedData)
    {
      $this->add();
    }
    else
    {
      $this->syncCategory();
    }
    $this->syncedObjectIds[] = $viaId;
    $this->syncCategoryRelation();
    $this->markObjectAsSynced();
  }

  /**
   * add category
   *
   * @return boolean 
   */
  protected function add()
  {
    $this->logger->logExecution();
    if (!$id = mo_viaebay__main::getInstance()->getInterface()->addCategory($this->syncObject))
    {
      return false;
    }

    $this->syncObject->oxcategories__mo_viaebay__via_id = new oxField($id);
    return $this->syncObject->save();
  }

  /**
   * sync category 
   */
  protected function syncCategory()
  {
    $needsUpdate = false;

    if ($this->objectFeedData['ForeignId'] != $this->syncObject->oxcategories__oxid->value)
    {
      $this->logger->info('syncCategory: ID mismatch');
      $needsUpdate = true;
    }

    if ($this->objectFeedData['Name'] != $this->syncObject->oxcategories__oxtitle->rawValue)
    {
      $this->logger->info('syncCategory: title mismatch');
      $needsUpdate = true;
    }

    if ($this->objectFeedData['IsRootLevel'] == $this->syncObject->mo_viaebay__hasParentCategory())
    {
      $this->logger->info('syncCategory: root-flag mismatch');
      $needsUpdate = true;
    }

    if ($needsUpdate)
    {
      $this->logger->logExecution();
      mo_viaebay__main::getInstance()->getInterface()->updateCategory($this->syncObject);
    }
  }

  /**
   * sync category relation
   *
   * @return boolean 
   */
  protected function syncCategoryRelation()
  {
    //delete async parent assignments
    $this->deleteAsyncCategoryRelations();

    //set parent category
    if (!$parentCategory = $this->syncObject->getParentCategory())
    {
      return;
    }
    
    //parent is not synced
    if (!$parentId = $parentCategory->oxcategories__mo_viaebay__via_id->value)
    {
      return false;
    }
    //check current parent
    if ($this->syncObject->oxcategories__mo_viaebay__via_parent_id->value != $parentId)
    {
      $this->syncObject->oxcategories__mo_viaebay__via_parent_id = new oxField(0);
    }
    //parent not set yet  
    if (!$this->syncObject->oxcategories__mo_viaebay__via_parent_id->value )
    {
      return $this->addRelation($parentId, $this->syncObject->oxcategories__mo_viaebay__via_id->value);
    }
    //add relation
    $parentRelations = $this->feed->getChildCategories($parentId);
    if (!in_array($this->syncObject->oxcategories__mo_viaebay__via_id->value, $parentRelations) )
    {
      return $this->addRelation($parentId, $this->syncObject->oxcategories__mo_viaebay__via_id->value);
    }
  }

  /**
   * delete async category relations
   */
  protected function deleteAsyncCategoryRelations()
  {
    $assignments = $this->feed->getParentAssignments($this->syncObject->oxcategories__mo_viaebay__via_id->value);
    $parentId    = null;

    if ($parentCategory = $this->syncObject->getParentCategory())
    {
      $parentId = $parentCategory->oxcategories__mo_viaebay__via_id->value;
    }

    foreach ($assignments as $assignedParentId)
    {
      if ($assignedParentId == $parentId)
      {
        continue;
      }
      $this->deleteRelation($assignedParentId, $this->syncObject->oxcategories__mo_viaebay__via_id->value);
    }
  }

  /**
   * add relation betwenn categories
   *
   * @param string $parentCategoryId
   * @param string $subCategoryId
   * @return boolean 
   */
  protected function addRelation($parentCategoryId, $subCategoryId)
  {
    $this->logger->logExecution();
    if (!mo_viaebay__main::getInstance()->getInterface()->setCategoryRelation($parentCategoryId, $subCategoryId))
    {

      return false;
    }
    $this->syncObject->oxcategories__mo_viaebay__via_parent_id = new oxField($parentCategoryId);
    return $this->syncObject->save();
  }

  /**
   * delete relation between categories
   *
   * @param string $parentCategoryId
   * @param string $subCategoryId
   * @return mixed 
   */
  protected function deleteRelation($parentCategoryId, $subCategoryId)
  {
    $this->logger->logExecution();
    return mo_viaebay__main::getInstance()->getInterface()->deleteCategoryRelation(
                    $parentCategoryId, $subCategoryId);
  }

  /**
   * delete async categories 
   */
  public function deleteAsyncData()
  {
    $ids = array();

    //collect ids first
    foreach ($this->feed->getData() as $id => $record)
    {
      if (!in_array($id, $this->syncedObjectIds))
      {
        $ids[] = $id;
      }
    }

    $this->deleteCategories($ids);
  }

  /**
   * delete categories in ascending order since via doesn't support deletion of assigned categories
   * 
   * @param arraz $ids 
   */
  protected function deleteCategories($ids)
  {
    if (!$ids)
    {
      return;
    }

    sort($ids);
    $ids = array_reverse(($ids));

    foreach ($ids as $id)
    {
      $this->logger->logExecution();
      mo_viaebay__main::getInstance()->getInterface()->deleteCategory($id);
    }
  }

  /**
   * mark object as synced 
   */
  protected function markObjectAsSynced()
  {
    $this->syncObject->oxcategories__mo_viaebay__last_sync = new oxField(date('Y-m-d H:i:s', time()));
    $this->syncObject->save();
  }

}