<?php

/**
 * mo_viaebay__sync_manager_orders
 * 
 * sync order with via ebay
 * 
 * @version   $Id: mo_viaebay__sync_manager_orders.php 220 2013-06-06 14:06:26Z $
 */

/**
 * sync order with via ebay
 * 
 * @package   mo_viaebay
 * @extend mo_viaebay__sync_manager 
 */
class mo_viaebay__sync_manager_orders extends mo_viaebay__sync_manager
{

    //via Mapping Array for deliverydata
    protected $VIAMappingArray = array('DE_Paket' => 'Paketversand',
        'DE_DeutschePostBrief' => 'Deutsche Post Brief',
        "DE_DHLPackchen" => "DHL Päckchen",
        "DE_Express" => "Expressversand",
        "DE_DHLPaket" => "DHL Paket",
        "DE_DPBuecherWarensendung" => "Deutsche Post Bücher-/Warensendung",
        "DE_HermesPaket" => "Hermes Paket",
        "DE_IloxxTransport" => "iloxx Transport",
        "DE_SonstigeDomestic" => "Sonstige 1-2 Werktage",
        "DE_Einschreiben" => "Einschreiben (inkl. aller Gebühren)",
        "DE_Nachname" => "Nachnahme (inkl. aller Gebühren)",
        "DE_SpecialDelivery" => "Sonderversand",
        "DE_SparversandAusDemAusland" => "Sparversand aus dem Ausland",
        "DE_StandardversandAusDemAusland" => "Standardversand aus dem Ausland",
        "DE_ExpressversandAusDemAusland" => "Expressversand aus dem Ausland",
        "DE_DPDExpress" => "DPD Express",
        "DE_UPSExpress" => "UPS Express",
        "DE_GLSPaket" => "GLS Paket",
        "DE_EbayHermesPaketShop2Shop" => "eBay Hermes Paket Shop2Shop",
        "DE_eBayHermesPaketSperrgutShop2Shop" => "eBay Hermes Paket Sperrgut Shop2Shop (Abgabe und Zustellung im Paketshop)",
        "DE_HermesPaketSperrgut" => "Hermes Paket Sperrgut",
        "DE_DPDClassic" => "DPD Classic",
        "DE_UPSStandard" => "UPS Standard",
        "DE_DeutschePostBriefInternational" => "Deutsche Post Brief International",
        "DE_DHLPackchenInternational" => "DHL Päckchen International",
        "DE_PaketInternational" => "Standard International Versand",
        "DE_ExpressInternational" => "Express International Versand",
        "DE_DHLPaketInternational" => "DHL Paket International",
        "DE_IloxxTransportInternational" => "iloxx Transport International",
        "DE_SonstigeInternational" => "Economy International Versand",
        "DE_HermesPaketInternational" => "Hermes Paket International",
        "DE_DeutschePostBriefIntlEcon" => "Deutsche Post Presse und Bücher Economy",
        "DE_DeutschePostBriefIntlPri" => "Deutsche Post Presse und Bücher Priority",
        "PromotionalShippingMethod" => "Versand zum Sonderpreis");

    /**
     * order submitted from via ebay
     *
     * @var array 
     */
    protected $viaOrder = null;

    /**
     * import order to be saved in oxid
     *
     * @var oxOrder 
     */
    protected $importOrder = null;

    /**
     * create new order (create basket and user first) from via-data
     * 
     * @param array $viaOrder 
     */
    public function import($viaOrder)
    {
        $this->setViaOrder($viaOrder);
        $this->validateViaOrder();

        $this->setImportOrder(oxNew('oxorder'));
        $this->setBasket(oxNew('oxbasket'));

        //force reinitialization of inputvalidator (reset errors)
        oxRegistry::set('oxInputValidator', null);
        
        $user = $this->createOrLoadUser();

        $this->basket->setBasketUser($user);

        //prevent discount calculation
        $this->basket->setDiscountCalcMode(false);
        $this->basket->setTotalDiscount(0);

        //set card information (prevent indefined notices)
        $this->basket->setCardId(null);
        $this->basket->setCardMessage(null);

        //add delivery costs
        $this->addDeliveryCosts();

        $this->addArticlesToBasket();

        oxRegistry::getSession()->setVariable('sess_challenge', oxUtilsObject::getInstance()->generateUID());

        //set via id
        $this->importOrder->oxorder__mo_viaebay__via_id = new oxField($this->viaOrder['Id']);

        //set other data

        if ($this->viaOrder['PaidDate']) {
            $this->importOrder->oxorder__oxpaid = new oxField($this->helper->convertViaDateToStandard($this->viaOrder['PaidDate']));
            $this->importOrder->oxorder__mo_viaebay__payment_state = new oxField(1);
        }

        $this->importOrder->oxorder__oxtransid = new oxField($this->viaOrder['PaymentTransactionId']);

        //finalize
        $this->importOrder->finalizeOrder($this->basket, $user, false);

        //check if order has been saved
        if (!$this->isOrderFinalized()) {
            throw new Exception('Order ' . $this->viaOrder['Id'] . ' was not saved to DB.');
        }

        //set order date
        $this->importOrder->mo_viaebay__setOrderDate($this->helper->convertViaDateToStandard($this->viaOrder['CheckoutCompletionDate']));

        oxRegistry::getSession()->setVariable('sess_challenge', null);

        //set payment
        if (!$paymentName = $this->getOxidPaymentName(new oxField($this->viaOrder['PaymentOption']))) {
            $paymentName = new oxField($this->viaOrder['PaymentOption']);
        }
        $this->importOrder->mo_viaebay__setPayment($paymentName);

        //add delivey information
        if (!$shippingService = $this->getOxidDeliveryType(new oxField($this->viaOrder['ShippingService']))) {
            $shippingService = new oxField($this->viaOrder['ShippingService']);
        }
        $this->importOrder->mo_viaebay__setShipping($shippingService);

        //change ShippingService to viaDescription
        $this->viaOrder['ShippingService'] = $this->mapVIAShippingService($this->viaOrder['ShippingService']);

        //store details
        $this->importOrder->mo_viaebay__saveOrderDetails($this->viaOrder);

        //save OXID ID to Via
        $this->saveOxidIdToVia();

        //reset sync dates for articles
        $this->resetSyncDatesForArticles();
    }

    /**
     * add delivery costs to order
     *
     * @return mixed 
     */
    protected function addDeliveryCosts()
    {
        if (empty($this->viaOrder['ShippingServiceCost'])) {
            return;
        }
        $shippingPrice = oxNew('oxprice');
        $shippingPrice->setPrice($this->viaOrder['ShippingServiceCost']);
        $this->basket->setDeliveryPrice($shippingPrice);
    }

    /**
     * save OXID ID as ForeignId
     */
    protected function saveOxidIdToVia()
    {
        mo_viaebay__main::getInstance()->getInterface()->updateOrder($this->importOrder->oxorder__mo_viaebay__via_id, 'ForeignOrderId', $this->importOrder->getId());
    }

    /**
     * check via data for integrity
     */
    protected function validateViaOrder()
    {
        // validate email
        if (empty($this->viaOrder['Address']['Email'])) {
            $this->handleError('invalid VIA order - missing email');
        }
        if (!oxRegistry::getUtils()->isValidEmail($this->viaOrder['Address']['Email'])) {
            $this->handleError('invalid VIA order - invalid email');
        }

        //order already exists ?
        if (empty($this->viaOrder['Id'])) {
            $this->handleError('invalid VIA order - missing ID');
        }

        $order = oxNew('oxorder');
        if ($order->mo_viaebay__loadByViaId($this->viaOrder['Id'])) {
            $this->handleError('invalid VIA order - order already exists');
        }

        if (!$this->checkOrderPositions()) {
            $this->handleError('invalid VIA order - empty ForeignId');
        }
    }

    /**
     * foreign ids in order positions
     *
     * @return boolean 
     */
    protected function checkOrderPositions()
    {
        foreach ($this->viaOrder['SalesOrderItems'] as $item) {
            if (empty($item['ForeignId'])) {
                return false;
            }
        }

        return true;
    }

    /**
     * log error and throw exception
     * 
     * @param string $message
     * @throws oxException
     */
    protected function handleError($message)
    {
        $exception = oxNew('oxException');
        $exception->setMessage($message);
        $this->logger->error($message, $this->viaOrder);
        throw $exception;
    }

    /**
     * setter
     * 
     * @param array $viaOrder 
     */
    public function setViaOrder($viaOrder)
    {
        $this->viaOrder = $viaOrder;
    }

    /**
     * setter
     * 
     * @param oxOrder $order 
     */
    public function setImportOrder(oxOrder $order)
    {
        $this->importOrder = $order;
    }

    /**
     * setter
     * 
     * @param oxBasket $basket 
     */
    public function setBasket(oxBasket $basket)
    {
        $this->basket = $basket;
    }

    /**
     * try to load user by email, create otherwise
     * 
     * @return oxuser
     */
    protected function createOrLoadUser()
    {
        $user = oxNew('oxuser');
        if ($id = $user->getIdByUserName($this->viaOrder['Address']['Email'])) {
            return $this->mo_viaebay__prepareUser($id);
        }

        //create user from via-data
        $billAddress = $this->getAddress();
        $username = $billAddress['oxuser__oxusername'];
        $user->checkValues($username, null, null, $billAddress, null);

        // setting values
        $user->oxuser__oxusername = new oxField($username, oxField::T_RAW);
        $user->setPassword(null);
        $user->oxuser__oxactive = new oxField(0);

        $user->createUser();
        $user->load($user->getId());
        $user->changeUserData($user->oxuser__oxusername->value, null, null, $billAddress, $delAddress);

        return $user;
    }

    /**
     * map via address to oxid address data
     * 
     * @return array 
     */
    protected function getAddress()
    {
        $viaAddress = $this->viaOrder['Address'];
        $prefix = 'oxuser__ox';

        $addressData = array();
        $addressData[$prefix . 'fname'] = $viaAddress['Name'];
        $addressData[$prefix . 'lname'] = $viaAddress['Surname'];

        $streetInfo = $this->getStreetAndNo($viaAddress['Street']);
        $addressData[$prefix . 'street'] = $streetInfo['street'];
        $addressData[$prefix . 'streetnr'] = $streetInfo['streetno'];

        $addressData[$prefix . 'addinfo'] = $viaAddress['Addition'];
        $addressData[$prefix . 'zip'] = $viaAddress['PostalCode'];
        $addressData[$prefix . 'city'] = $viaAddress['Town'];
        $addressData[$prefix . 'countryid'] = $this->getCountryId($viaAddress['Country']);
        $addressData[$prefix . 'sal'] = $viaAddress['Salutation'] ? $viaAddress['Salutation'] : 'MR';
        $addressData[$prefix . 'fon'] = $viaAddress['Phone'] ? $viaAddress['Phone'] : '-';
        $addressData[$prefix . 'username'] = $viaAddress['Email'];
        $addressData[$prefix . 'company'] = $viaAddress['Company'];

        $addressData = $this->setRequiredFields($addressData, $prefix);

        return $addressData;
    }

    protected function setRequiredFields($addressData, $prefix)
    {
        //add dummy data for other required fields
        if (!$requiredFields = $this->oxConfig->getConfigParam('aMustFillFields')) {
            return $addressData;
        }

        foreach ($requiredFields as $field) {
            if (strpos($field, $prefix) !== 0) {
                continue;
            }

            if (!empty($addressData[$field])) {
                continue;
            }

            $addressData[$field] = '-';
        }

        return $addressData;
    }

    /**
     * workaround: cut off last word and set as streetno (OXID demands streetno)
     * 
     * @param string $viaStreetData
     * @return array 
     */
    protected function getStreetAndNo($viaStreetData)
    {
        $streetData = array('street' => '', 'streetno' => '-');

        if (!preg_match('#^(.+) (\w+)$#', $viaStreetData, $matches)) {
            $streetData['street'] = $viaStreetData;
            return $streetData;
        }

        $streetData['street'] = $matches[1];
        $streetData['streetno'] = $matches[2];
        return $streetData;
    }

    /**
     * add articles to basket and set via properties 
     */
    protected function addArticlesToBasket()
    {
        foreach ($this->viaOrder['SalesOrderItems'] as $item) {
            if (empty($item['ProductVariation']['Sku'])) {
                $this->basket->addToBasket($item['ForeignId'], $item['Amount']);
            } else {
                $this->basket->addToBasket($item['ProductVariation']['Sku'], $item['Amount']);
            }
        }

        $this->setArticleViaProperties();
    }

    /**
     * set properties on virtual basketitems' articles
     * (remap them in mo_viaebay__oxbasketitem::getArticle)
     */
    protected function setArticleViaProperties()
    {
        $basketContents = $this->basket->getContents();

        foreach ($this->viaOrder['SalesOrderItems'] as $item) {
            $viaId = $item['Id'];
            $viaPrice = $item['Price'];

            if (!empty($item['ProductVariation']['Sku'])) {
                $itemKey = $this->basket->getItemKey($item['ProductVariation']['Sku']);
            } else {
                $itemKey = $this->basket->getItemKey($item['ForeignId']);
            }

            $article = $basketContents[$itemKey]->getArticle();

            //set price - VIA always works with brutto-prices => convert to netto price if necessary
            $price = oxNew('oxPrice');
            $price->setBruttoPriceMode();
            $price->setPrice($viaPrice, $article->getPrice()->getVat());
            $priceFnc = $this->oxConfig->getConfigParam('blEnterNetPrice') ? 'getNettoPrice' : 'getBruttoPrice';
            $article->oxarticles__oxprice = new oxField($price->$priceFnc());

            //set via-id
            $article->oxarticles__mo_viaebay__via_id = new oxField($viaId);

            $basketContents[$itemKey]->mo_viaebay__setArticle($article);
        }

        $this->basket->mo_viaebay__setBasketContents($basketContents);
        $this->basket->onUpdate();
        $this->basket->calculateBasket();
    }

    /**
     * load country ID by ISO-code
     * @param string $countryCode
     * @return string 
     */
    protected function getCountryId($countryCode)
    {
        $oxCountry = oxNew('oxcountry');
        return $oxCountry->getIdByCode($countryCode);
    }

    /**
     * prepare user
     *
     * @param string $id
     * @return oxuser 
     */
    protected function mo_viaebay__prepareUser($id)
    {
        $user = oxNew('oxuser');

        $user->load($id);

        //map address data to user
        $billAddress = $this->getAddress();

        foreach ($billAddress as $key => $value) {
            $user->$key = new oxField($value);
        }
        return $user;
    }

    /**
     * reset sync dates for ordered articles
     */
    protected function resetSyncDatesForArticles()
    {
        foreach ($this->importOrder->getOrderArticles() as $orderArticle) {
            //reload from DB since orderArticle's article is deprecated due to stock changes
            if (!$article = $this->mo_viaebay__getBaseArticle($orderArticle->oxorderarticles__oxartid->value)) {
                continue;
            }

            $article->oxarticles__mo_viaebay__last_sync = new oxField('0000-00-00');
            $article->save();
        }
    }

    protected function mo_viaebay__getBaseArticle($articleId)
    {
        $article = oxNew('oxarticle');
        if (!$article->load($articleId)) {
            return null;
        }

        //has parent ?
        if ($parentArticle = $article->getParentArticle()) {
            return $parentArticle;
        }

        return $article;
    }

    protected function isOrderFinalized()
    {
        $orderTbl = getViewName('oxorder');
        $oxDb = oxDb::getDb();
        $sql = "SELECT OXID FROM " . $orderTbl . " WHERE OXID = " . $oxDb->quote($this->importOrder->getId());
        return $oxDb->getOne($sql);
    }

    protected function getOxidPaymentName($paymentName)
    {
        $db = oxdb::getDb();
        $sql = "SELECT oxid
            FROM " . getViewName('oxpayments') . " 
            WHERE MO_VIAEBAY__PAYMENT_NAME = " . $db->quote($paymentName) . " ";

        return oxDb::getDb()->GetOne($sql);
    }

    protected function getOxidDeliveryType($deliveryType)
    {
        $db = oxdb::getDb();
        $sql = "SELECT oxid
            FROM " . getViewName('oxdeliveryset') . " 
            WHERE MO_VIAEBAY__SHIPPING_NAME = " . $db->quote($deliveryType) . " ";

        return oxDb::getDb()->GetOne($sql);
    }

    public function mapVIAShippingService($shippingService)
    {
        return $this->VIAMappingArray[$shippingService];
    }

}
