<?php

/**
 * mo_viaebay__status
 * 
 * Check VIAebay status
 * 
 * @version   $Id: mo_viaebay__status.php 217 2013-04-29 10:02:57Z $
 */

/**
 * Check VIAebay status
 * 
 * @package   mo_viaebay
 * @extend oxAdminView 
 */
class mo_viaebay__status extends oxAdminView
{

  /**
   * class template.
   * @var string
   */
  protected $_sThisTemplate = 'mo_viaebay__status.tpl';

  /**
   * render
   * 
   * @extend render
   * @return string
   */
  public function render()
  {
    parent::render();

    return $this->_sThisTemplate;
  }

  /**
   * download chosen logfile-archive
   *
   * @return mixed boolean / file 
   */
  public function mo_viaebay__download()
  {
    $oxConfig = $this->getConfig();
    $filename = $oxConfig->getRequestParameter('filename');

    if (!$this->mo_viaebay__validateFileName($filename))
    {
      return false;
    }

    $filename = $oxConfig->getLogsDir() . $filename;

    if (!$this->mo_viaebay__checkIfFileExists($filename))
    {
      return false;
    }

    header('Pragma: public');
    header('Cache-Control: private', false);
    header('Content-Type: application/zip');

    header('Content-Disposition: attachment; filename="' . basename($filename) . '";');
    header('Content-Transfer-Encoding: binary');
    header('Content-Length: ' . filesize($filename));

    readfile($filename);

    exit;
  }

  /**
   * return array with filenames and timstamps
   *
   * @param array $logFileList
   * @return array 
   */
  protected function mo_viaebay__handleLogFiles($logFileList)
  {
    $oxConfig = $this->getConfig();
    $logFiles = array();

    foreach ($logFileList as $file)
    {
      $fileName = basename($file);

      if (strstr($fileName, 'mo_viaebay'))
      {
        $logFiles[] = array('filetime' => filectime($oxConfig->getLogsDir() . $fileName), 'filename' => $fileName);
      }
    }

    return $logFiles;
  }

  /**
   * filename should match via logfile pattern
   *
   * @param string $filename
   * @return boolean 
   */
  protected function mo_viaebay__validateFileName($filename)
  {
    if (preg_match('#^mo_viaebay_[_\w]+\.log\.zip$#', $filename))
    {

      return true;
    }

    $sError = 'MO_VIAEBAY_NO_VIAEBAY_LOGFILE';
    $oEx    = oxNew("oxExceptionToDisplay");
    $oEx->setMessage($sError);
    oxRegistry::get("oxUtilsView")->addErrorToDisplay($oEx, false);

    return false;
  }

  /**
   * check if file exists
   *
   * @param string $filename
   * @return boolean 
   */
  protected function mo_viaebay__checkIfFileExists($filename)
  {
    if (file_exists($filename))
    {
      return true;
    }
    $sError = 'MO_VIAEBAY_FILE_DOESNOT_EXIST';
    $oEx    = oxNew("oxExceptionToDisplay");
    $oEx->setMessage($sError);
    oxRegistry::get("oxUtilsView")->addErrorToDisplay($oEx, false);
    return false;
  }

  /**
   * get current logfile
   *
   * @return string 
   */
  public function mo_viaebay__getActiveLogfile()
  {
    $oxConfig = $this->getConfig();

    $logFile = $oxConfig->getLogsDir() . "mo_viaebay.log";
    if (file_exists($logFile)) {
        $log = $this->parseLog($logFile);
        return array_reverse($this->filterLog($log));
    }

    return array();
  }
  
    /**
     * filter log (afterwards due to multiline messages)
     * @param type $log
     */
    protected function filterLog($log)
    {
        $viaConfig = $this->mo_viaebay__getConfig();
        $logLevels = array(0 => 'DEBUG', 1 => 'INFO', 2 => 'ERROR');
        $logDates = array(0 => 'day', 1 => 'week', 2 => 'month');

        $logLevelFilter = empty($viaConfig['loglevel']) ? 0 : $viaConfig['loglevel'];
        $logDateFilter = empty($viaConfig['logdate']) ? 0 : $viaConfig['logdate'];
        $logInterval = new DateTime('-1 ' . $logDates[$logDateFilter]);

        foreach ($log as $key => $entry) {
            $logDate = new DateTime($entry['date']);
            if ($logDate < $logInterval) {
                unset($log[$key]);
            } elseif (array_search($entry['level'], $logLevels) < $logLevelFilter) {
                unset($log[$key]);
            } elseif (!empty($viaConfig['logsearch']) && (stripos($entry['message'], $viaConfig['logsearch']) === false)) {
                unset($log[$key]);
            }
        }
        return $log;
    }

    /**
   * parse logfile
   * @param type $logFile
   * @return type
   */
  protected function parseLog($logFile) {
      $viaConfig = $this->mo_viaebay__getConfig();
      $fh = fopen($logFile, 'r');
      $log = array();
      $id = null;
      while ($line = fgets($fh)) {
          $line = trim($line);
          if (!$line) {
              continue;
          }
          
          if(preg_match('#^([\d]{4}-[\d]{2}-[\d]{2} [\d]{2}:[\d]{2}:[\d]{2}) (.*?): (.*)#', $line, $matches)) {
              $id = uniqid();
              $log[$id] = array('date' => $matches[1], 'level' => $matches[2], 'message' => $matches[3]);
          } elseif (!$id) {
              continue;
          } else {
              if(!empty($viaConfig['loglong']) || (empty($viaConfig['loglong']) && strlen($log[$id]['message']) < 1000)) {
                $log[$id]['message'] .= "\n" . $line;
              }
          }
      }
      fclose($fh);
      return $log;
  }

  /**
   * get logfile archives
   *
   * @return array 
   */
  public function mo_viaebay__getLogFiles()
  {
    $oxConfig = $this->getConfig();

    $logFileList = glob($oxConfig->getLogsDir() . 'mo_viaebay*.zip');
    $logFiles    = array();

    $logFiles = $this->mo_viaebay__handleLogFiles($logFileList);

    arsort($logFiles);

    return array_slice($logFiles, 0, 10);
  }

  /**
   * get viaebay config
   *
   * @return array 
   */
  public function mo_viaebay__getConfig()
  {
    $viaMain = mo_viaebay__main::getInstance();
    return $viaMain->getViaebayConfig();
  }

  /**
   * get viaebay tasks
   *
   * @return array 
   */
  public function mo_viaebay__getTasks()
  {
    return mo_viaebay__main::getInstance()->getTasks();
  }

    /**
     * save config
     * 
     * @extend save
     * @return void
     */
    public function save()
    {
        $viaMain = mo_viaebay__main::getInstance();
        $viaConfig = $viaMain->getViaebayConfig();
        $oxConfig = $this->getConfig();

        $params = $oxConfig->getRequestParameter('mo_viaebay__config');
        $viaConfig = array_merge($viaConfig, $params);

        $viaMain->setViaebayConfig($viaConfig);
        $viaMain->saveViaConfig();
    }

}