<?php

/**
 * mo_viaebay__oxarticle
 * 
 * add via functions to oxarticle
 * 
 * @version   $Id: mo_viaebay__oxarticle.php 213 2013-03-15 11:29:15Z $
 */

/**
 * add via functions to oxarticle
 * 
 * @package   mo_viaebay
 * @extend oxarticle  
 */
class mo_viaebay__oxarticle extends mo_viaebay__oxarticle_parent
{

    /**
     * get viaebay export stock
     *
     * @return string 
     */
    public function mo_viaebay__getStock()
    {
        //return 0 for variant parents / not buyable products
        if (!$this->isBuyable()) {
            return 0;
        }

        $viaConfig = mo_viaebay__main::getInstance()->getViaebayConfig();

        if ((int) $this->oxarticles__oxstockflag->value === 4) {
            return $viaConfig['stock_export'];
        }
        if ($viaConfig['stock_export'] > $this->oxarticles__oxstock->value) {
            return $this->oxarticles__oxstock->value;
        }

        return $viaConfig['stock_export'];
    }

    /**
     * get product type
     *
     * @return int 
     */
    public function mo_viaebay__getProductType()
    {
        if ($this->_hasAnyVariant()) {
            return mo_viaebay__main::MO_VIAEBAY__PRODUCT_TYPE__VARIATION_SET;
        }

        return mo_viaebay__main::MO_VIAEBAY__PRODUCT_TYPE__SINGLE_PRODUCT;
    }

    /**
     * get via category ids
     *
     * @return array 
     */
    public function mo_viaebay__getViaCategoryIds()
    {
        if (!$categoryIds = $this->getCategoryIds()) {
            return array();
        }
        $oxConfig = $this->getConfig();
        $shopId = $oxConfig->getShopID();
        $db = oxdb::getDb();
        $sql = "SELECT mo_viaebay__via_id
            FROM " . getViewName('mo_viaebay__category2via') . "
            WHERE OXCATNID IN ('" . implode("', '", $categoryIds) . "') AND (OXSHOPID = " . $db->quote($shopId) . ") 
            AND (mo_viaebay__via_id IS NOT NULL)";

        return oxDb::getDb()->GetCol($sql);
    }

    /**
     * use special viaeBay-longDescription for viaEbay articles, if checkbox in the configuration is checked 
     * In Syncmode, we save a new LongDescription with attributes, afterwards we just want to work with the OxidLongDescription
     *  
     * @return object $oField field object
     */
    public function getLongDescription($syncmode = false)
    {
        if (!defined('MO_VIAEBAY__IS_SCRIPT_CONTEXT')) {
            return parent::getLongDescription();
        }


        $viaLongDesc = $this->mo_viaebay__getLongDescription();
        if ((empty($viaLongDesc)) || ($syncmode)) {
            return parent::getLongDescription();
        }
        $this->oLongDesc = new oxField();
        $this->oLongDesc->setValue($viaLongDesc, oxField::T_RAW);
        return $this->oLongDesc;
    }

    /**
     * get viaeBay longdescription for an article from db 
     * 
     * @return string
     */
    public function mo_viaebay__getLongDescription()
    {
        $result = "";
        $oxConfig = oxRegistry::getConfig();
        $params = $oxConfig->getConfigParam('mo_viaebay__config');
        $reference_table = $params['description_tablename'];
        $reference_column = $params['description_coloumnname'];
        $articleId = $params['article_id'];
        $sOxid = $this->getId();
        $sViewName = getViewName('oxartextends', $this->getLanguage());

        $oDb = oxDb::getDb();
        $result = $oDb->getOne("select $reference_column from $reference_table where $articleId = " . $oDb->quote($sOxid));
        return $result;
    }

    /**
     * prevent saving oxlongdescription in via-context because of temporary change in sync-manager
     * 
     * @return null
     */
    protected function _saveArtLongDesc()
    {
        if (!defined('MO_VIAEBAY__IS_SCRIPT_CONTEXT')) {
            return parent::_saveArtLongDesc();
        }
    }

    public function load($sOXID)
    {
        $result = parent::load($sOXID);
        //do not load via data in frontend (via cronjobs have also admin-mode enabled)
        if (!isAdmin()) {
            return $result;
        }
        $this->mo_viaebay__assignViaData();
        return $result;
    }

    /**
     * additional saving of category2via-Data
     * @return bool
     */
    public function save()
    {
        if (!defined('MO_VIAEBAY__IS_SCRIPT_CONTEXT')) {
            $this->resetLastSyncDate();
        }
        $this->mo_viaebay__saveViaData();
        return parent::save();
    }

    /**
     * reset sync date (for all subshops)
     */
    protected function resetLastSyncDate()
    {
        $oxDb = oxDb::getDb();
        $sql = "UPDATE mo_viaebay__articles2via 
                SET  
                MO_VIAEBAY__LAST_SYNC = 0,
                MO_VIAEBAY__LAST_MPSYNC = 0
                WHERE 
                OXARTID = " . $oxDb->quote($this->getId()) . " OR 
                OXARTID = " . $oxDb->quote($this->getParentId());
        //set property, otherwise date gets restored again in mo_viaebay__saveViaData
        $this->oxarticles__mo_viaebay__last_sync = 0;
        oxDb::getDb()->execute($sql);
    }

    protected function mo_viaebay__assignViaData()
    {
        $viaData = oxNew('mo_viaebay__articles2via');
        //create and load with DB-default values if viadata doesn't exist
        if (!$viaData->load($this->getId())) {
            $viaData->save();
            $viaData->load($this->getId());
        }
        foreach ($viaData as $key => $v) {
            if (strpos($key, 'mo_viaebay__articles2via__mo_viaebay') === 0) {
                $fieldName = str_replace('mo_viaebay__articles2via__', 'oxarticles__', $key);
                $this->$fieldName = $v;
            }
        }
    }
    
    /**
     * Saving Data in DB-Table mo_viaebay__category2via
     */
    protected function mo_viaebay__saveViaData()
    {
        $viaData = oxNew('mo_viaebay__articles2via');
        $viaData->load($this->getId());
        foreach ($this as $key => $v) {
            if (strpos($key, 'oxarticles__mo_viaebay__') === 0) {
                $fieldName = str_replace('oxarticles__', 'mo_viaebay__articles2via__', $key);
                $viaData->$fieldName = $v;
            }
        }
        return $viaData->save();
    }

    public function mo_viaebay__getManufacturername()
    {
        $oManufacturer = oxNew('oxmanufacturer');
        if (!$sManufacturerId = $this->oxarticles__oxmanufacturerid->value) {
            return "";
        }
        $oManufacturer->load($sManufacturerId);
        $manufacturername = $oManufacturer->oxmanufacturers__oxtitle->value;
        return $manufacturername;
    }

    public function mo_viaebay__getEanForVariant()
    {
        if ($ean = $this->oxarticles__oxean->value) {
            return $ean;
        }
        if ($parent = $this->getParentArticle()) {
            return $parent->oxarticles__oxean->value;
        }
        return "";
    }

    public function mo_viaebay__getMpnForVariant()
    {
        if ($mpn = $this->oxarticles__oxmpn->value) {
            return $mpn;
        }
        if ($parent = $this->getParentArticle()) {
            return $parent->oxarticles__oxmpn->value;
        }
        return "";
    }

    public function checkForStock($dAmount, $dArtStockAmount = 0)
    {
        if (!defined('MO_VIAEBAY__IS_SCRIPT_CONTEXT')) {
            return parent::checkForStock($dAmount, $dArtStockAmount);
        }
        return true;
    }

    public function mo_viaebay__loadByViaId($viaId)
    {
        $articles2via = oxNew('mo_viaebay__articles2via');
        if (!$articles2via->loadByViaId($viaId)) {
            return false;
        }
        return $this->load($articles2via->mo_viaebay__articles2via__oxartid->value);
    }
    
    /**
     * flush object
     * (main leak seems to be $this->_oAmountPriceList)
     */
    public function mo_viaebay__flush()
    {
        foreach (array_keys(get_object_vars($this)) as $var) {
            unset($this->$var);
        }
        
        unset ($this);
    }
}
