<?php

/**
 * mo_viaebay__oxarticlelist
 * 
 * add via ebay functionality to oxarticlelist
 * 
 * @version   $Id: mo_viaebay__oxarticlelist.php 211 2013-03-15 11:13:38Z $
 */

/**
 * add via ebay functionality to oxarticlelist
 * 
 * @package   mo_viaebay
 * @extend oxarticlelist 
 */
class mo_viaebay__oxarticlelist extends mo_viaebay__oxarticlelist_parent
{

    /**
     * get export articles
     *
     * @return oxarticlelist 
     */
    public function mo_viaebay__getList()
    {
        $dummy = oxNew('oxarticle');
        $articleTable = $dummy->getViewName();
        $oxDb = oxDb::getDb();

        $objectId = 'mo_viaebay_' . $this->getConfig()->getShopId();
        $catRefTable = getViewName('oxobject2category');
        $selectListRefTable = getViewName('oxobject2selectlist');
        $art2viaTable = getViewName('mo_viaebay__articles2via');
        $sql = "SELECT DISTINCT " . $dummy->getSelectFields() . "
            FROM " . $articleTable . "
            INNER JOIN " . $catRefTable . "
            ON " . $articleTable . ".OXID = " . $catRefTable . ".OXOBJECTID 
            LEFT JOIN " . $art2viaTable . " 
            ON " . $articleTable . ".OXID = " . $art2viaTable . ".OXARTID 
            LEFT JOIN " . $selectListRefTable . "
            ON " . $articleTable . ".OXID = " . $selectListRefTable . ".OXOBJECTID ";
        
        if ($this->getConfig()->isMall()) {
            $sql .= " LEFT JOIN oxfield2shop
                    ON " . $articleTable . ".OXID = oxfield2shop.OXARTID AND 
                    oxfield2shop.OXSHOPID = " . $oxDb->quote($this->getConfig()->getShopId()) . " ";
        }
        
        $sql .= "
            WHERE
            " . $dummy->getSqlActiveSnippet() . "
            " . $this->mo_viaebay__getFilterConditions($articleTable) . "  
            " . $this->mo_viaebay__getAssignedCategories($objectId, $catRefTable) . "
            " . $this->mo_viaebay__getAssignedManufacturers($objectId, $articleTable) . "
            " . $this->mo_viaebay__getExcludeConditions($articleTable, $selectListRefTable) . "
            " . $this->mo_viaebay__getSyncDateCondition() . "
            ORDER BY " . $art2viaTable . ".MO_VIAEBAY__LAST_SYNC ASC
            LIMIT " . mo_viaebay__main::MO_VIAEBAY__TICK_SIZE; //set tick-size = 20 => otherwise URL with filter params too long
        $this->selectString($sql);
        return $this;
    }
    
    /**
     * add conditions for article sync depending on article's oxtimestamp & sync-dates
     * @return string
     */
    protected function mo_viaebay__getSyncDateCondition()
    {
        $articleTable = getViewName('oxarticles');
        $art2viaTable = getViewName('mo_viaebay__articles2via');
        
        $sql = " AND ( " . $art2viaTable . ".MO_VIAEBAY__LAST_SYNC <  "  . $articleTable . ".OXTIMESTAMP "; 
        if ($this->getConfig()->isMall()) {
            $sql .= " OR " . $art2viaTable . ".MO_VIAEBAY__LAST_SYNC <  oxfield2shop.OXTIMESTAMP ";
        }
        $sql .= " OR " . $art2viaTable . ".MO_VIAEBAY__LAST_SYNC IS NULL) ";
        
        return $sql;
    }

    /**
     * get simple list of export articles
     * is Shop Id important and do we have to sort and check the viaId
     * 
     * @param Boolean $checkViaId
     * @return array 
     */
    public function mo_viaebay__getSimpleList($checkViaId = false)
    {
        $db = oxDb::getDb();

        $dummy = oxNew('oxarticle');
        $articleTable = $dummy->getViewName();

        $objectId = 'mo_viaebay_' . $this->getConfig()->getShopId();

        $catRefTable = getViewName('oxobject2category');
        $selectListRefTable = getViewName('oxobject2selectlist');
        $art2viaTable = getViewName('mo_viaebay__articles2via');

        $sql = "SELECT DISTINCT " .
                $articleTable . ".OXID, " .
                $articleTable . ".OXTITLE, " .
                $articleTable . ".OXARTNUM, " .
                $art2viaTable . ".MO_VIAEBAY__VIA_ID
            FROM " . $articleTable . "
            INNER JOIN " . $catRefTable . "
            ON " . $articleTable . ".OXID = " . $catRefTable . ".OXOBJECTID 
            LEFT JOIN " . $art2viaTable . "
            ON " . $articleTable . ".OXID = " . $art2viaTable . ".OXARTID 
            LEFT JOIN " . $selectListRefTable . "
            ON " . $articleTable . ".OXID = " . $selectListRefTable . ".OXOBJECTID 
            WHERE 
            " . $dummy->getSqlActiveSnippet() . "
            " . $this->mo_viaebay__getFilterConditions($articleTable) . "  
            " . $this->mo_viaebay__getAssignedCategories($objectId, $catRefTable) . "
            " . $this->mo_viaebay__getAssignedManufacturers($objectId, $articleTable) . "
            " . $this->mo_viaebay__getExcludeConditions($articleTable, $selectListRefTable, $checkViaId);
        if ($checkViaId) {
            $sql .= " ORDER BY  " . $art2viaTable . ".MO_VIAEBAY__LAST_MPSYNC ASC";
        } else {
            $sql .= " ORDER BY  " . $articleTable . ".OXTITLE ASC";
        }

        $result = $db->getAssoc($sql);

        return $result;
    }

    /**
     * fetch all via ids from active articles
     * 
     * @return array 
     */
    public function mo_viaebay__getAllViaEbayIdsFromActiveArticles()
    {
        $ids = array();
        $list = $this->mo_viaebay__getSimpleList();

        foreach ($list as $article) {
            $ids[] = $article[2];
        }
        return $ids;
    }
    
    /**
     * fetch all via ids in mo_viaebay__articles2via table
     */
    public function mo_viaebay__getAllViaEbayIds()
    {
        $oxDb = oxDb::getDb();
        $sql = "SELECT MO_VIAEBAY__VIA_ID
                FROM mo_viaebay__articles2via
                WHERE 
                OXSHOPID = " . $oxDb->quote($this->getConfig()->getShopId());
        return $oxDb->getCol($sql);
    }

    /**
     * apply article filter to select string
     *
     * @param string $articleTable
     * @return string 
     */
    protected function mo_viaebay__getFilterConditions($articleTable)
    {
        $viaebayConfig = mo_viaebay__main::getInstance()->getViaebayConfig();
        $filterConfig = $viaebayConfig['mo_viaebay__filter'];
        $filterString = '';

        if (!empty($filterConfig['min_price'])) {
            $filterString .= " AND " . $articleTable . ".OXPRICE >= " . $filterConfig['min_price'];
        }

        if (!empty($filterConfig['max_price'])) {
            $filterString .= " AND " . $articleTable . ".OXPRICE <= " . $filterConfig['max_price'];
        }

        if ($filterConfig['article_image'] == 1) {
            $filterString .= " AND " . $articleTable . ".OXPIC1 != '' ";
        }

        //only for non-variant articles
        if (!empty($filterConfig['min_stock'])) {
            $filterString .= " AND (" . $articleTable . ".OXSTOCK >= " . $filterConfig['min_stock'] .
                    " OR " . $articleTable . ".OXVARSTOCK >= " . $filterConfig['min_stock'];
            if ($filterConfig['contract_warehouse'] == 1) {
                $filterString .= " OR " . $articleTable . ".OXSTOCKFLAG = 4";
            }
            $filterString .=") ";
        }

        return $filterString;
    }

    /**
     * check if cateogories of export articles are flagged for export
     *
     * @param string $objectId
     * @param string $refTable
     * @return string 
     */
    protected function mo_viaebay__getAssignedCategories($objectId, $refTable)
    {
        $db = oxDb::getDb();

        $sql = "SELECT OXCATNID
            FROM " . getViewName('mo_viaebay__assigned_categories') . "
            WHERE 
            OXOBJECTID = " . $db->quote($objectId);
        $assignedCatIds = $db->GetCol($sql);

        if (!$assignedCatIds) {
            return '';
        }

        $sql = " AND " . $refTable . ".OXCATNID IN ('" . implode("', '", $assignedCatIds) . "')";

        return $sql;
    }

    /**
     * check if manufactureres of export articles are flagged for export
     *
     * @param string $objectId
     * @param string $articleTable
     * @return string 
     */
    protected function mo_viaebay__getAssignedManufacturers($objectId, $articleTable)
    {
        $db = oxDb::getDb();

        $sql = "SELECT OXMANUFACTURERID
            FROM " . getViewName('mo_viaebay__assigned_manufacturers') . "
            WHERE 
            OXOBJECTID = " . $db->quote($objectId);
        $assignedManIds = $db->GetCol($sql);

        if (!$assignedManIds) {
            return '';
        }

        $sql = " AND " . $articleTable . ".OXMANUFACTURERID IN ('" . implode("', '", $assignedManIds) . "') ";

        return $sql;
    }

    /**
     * add conditions for unsupported / unflagged articles
     * 
     * @param string $articleTable
     * @param string $selectListRefTable
     * @return string 
     */
    protected function mo_viaebay__getExcludeConditions($articleTable, $selectListRefTable, $checkViaId = false)
    {
        $viaebayConfig = mo_viaebay__main::getInstance()->getViaebayConfig();
        $articles2via = getViewName('mo_viaebay__articles2via');
        $db = oxDb::getDb();
        $shopId = $this->getConfig()->getShopId();
        //unflagged
        $sql = " AND ((" . $articles2via . ".MO_VIAEBAY__ALLOW_EXPORT = TRUE) OR (" . $articles2via . ".MO_VIAEBAY__ALLOW_EXPORT IS NULL))";

        //exclude articles with selectlists
        if (empty($viaebayConfig['export_selectlist_articles'])) {
            $sql .= " AND " . $selectListRefTable . ".OXID IS NULL ";
        }

        //variant-articles without variant-property-title
        $sql .= " AND (" . $articleTable . ".OXVARSTOCK = 0 OR  " . $articleTable . ".OXVARNAME != '') ";

        //exclude variants
        $sql .= " AND " . $articleTable . ".OXPARENTID = '' "; //sometimes variants might be assigned to categories...
        
        //inventory update only on articles with viaID
        if ($checkViaId) {
            $sql .= " AND " . $articles2via . ".MO_VIAEBAY__VIA_ID IS NOT NULL ";
            $sql .= " AND " . $articles2via . ".MO_VIAEBAY__VIA_ID != 0 ";
        }

        //include shopID
        $sql .= " AND ((" . $articles2via . ".OXSHOPID = " . $db->quote($shopId) . ") OR ( " . $articles2via . ".OXSHOPID IS NULL ))";

        return $sql;
    }

}
