<?php

/**
 * mo_viaebay__oxbasketitem
 * 
 * add via ebay functionality to oxbasketitem
 * 
 * @version   $Id: mo_viaebay__oxbasketitem.php 139 2012-12-10 16:51:05Z $
 */

/**
 * add via ebay functionality to oxbasketitem
 * 
 * @package   mo_viaebay
 * @extend oxbasketitem 
 */
class mo_viaebay__oxbasketitem extends mo_viaebay__oxbasketitem_parent
{

    /**
     * keep via price when importing via orders to oxid
     *
     * @param boolean $blCheckProduct
     * @param string $sProductId
     * @param boolean $blDisableLazyLoading
     * @return oxArticle 
     */
    public function getArticle($blCheckProduct = true, $sProductId = null, $blDisableLazyLoading = false)
    {
        if (!defined('MO_VIAEBAY__IS_SCRIPT_CONTEXT')) {
            return parent::getArticle($blCheckProduct, $sProductId, $blDisableLazyLoading);
        }

        if ($this->_oArticle === null) {
            return $this->mo_viaebay__changedParentcall($blCheckProduct, $sProductId, $blDisableLazyLoading);
        }

        //keep via price
        $viaPrice = $this->_oArticle->oxarticles__oxprice->value;
        $viaId = $this->_oArticle->oxarticles__mo_viaebay__via_id->value;

        $this->_oArticle = $this->mo_viaebay__changedParentcall($blCheckProduct, $sProductId, $blDisableLazyLoading);

        $this->_oArticle->oxarticles__oxprice = new oxField($viaPrice);
        $this->_oArticle->oxarticles__mo_viaebay__via_id = new oxField($viaId);
        return $this->_oArticle;
    }
    /**
     * getArticleParentCall without IsBuyableCheck
     * 
     * @param type $blCheckProduct
     * @param type $sProductId
     * @param type $blDisableLazyLoading
     */
    public function mo_viaebay__changedParentcall($blCheckProduct, $sProductId, $blDisableLazyLoading)
    {
         if ($this->_oArticle === null || (!$this->_oArticle->isOrderArticle() && $blDisableLazyLoading)) {
            $sProductId = $sProductId ? $sProductId : $this->_sProductId;
            if (!$sProductId) {
                //this exception may not be caught, anyhow this is a critical exception
                /** @var oxArticleException $oEx */
                $oEx = oxNew('oxArticleException');
                $oEx->setMessage('EXCEPTION_ARTICLE_NOPRODUCTID');
                throw $oEx;
            }

            $this->_oArticle = oxNew('oxarticle');
            // #M773 Do not use article lazy loading on order save
            if ($blDisableLazyLoading) {
                $this->_oArticle->modifyCacheKey('_allviews');
                $this->_oArticle->disableLazyLoading();
            }

            // performance:
            // - skipping variants loading
            // - skipping 'ab' price info
            // - load parent field
            $this->_oArticle->setNoVariantLoading(true);
            $this->_oArticle->setLoadParentData(true);
            if (!$this->_oArticle->load($sProductId)) {
                /** @var oxNoArticleException $oEx */
                $oEx = oxNew('oxNoArticleException');
                $oLang = oxRegistry::getLang();
                $oEx->setMessage(sprintf($oLang->translateString('ERROR_MESSAGE_ARTICLE_ARTICLE_DOES_NOT_EXIST', $oLang->getBaseLanguage()), $sProductId));
                $oEx->setArticleNr($sProductId);
                $oEx->setProductId($sProductId);
                throw $oEx;
            }

            // cant put not visible product to basket (M:1286)
            if ($blCheckProduct && !$this->_oArticle->isVisible()) {
                /** @var oxNoArticleException $oEx */
                $oEx = oxNew('oxNoArticleException');
                $oLang = oxRegistry::getLang();
                $oEx->setMessage(sprintf($oLang->translateString('ERROR_MESSAGE_ARTICLE_ARTICLE_DOES_NOT_EXIST', $oLang->getBaseLanguage()), $this->_oArticle->oxarticles__oxartnum->value));
                $oEx->setArticleNr($sProductId);
                $oEx->setProductId($sProductId);
                throw $oEx;
            }

        }

        return $this->_oArticle;
    }

    /**
     * set article
     *
     * @param oxArticle $article 
     */
    public function mo_viaebay__setArticle(oxArticle $article)
    {
        $this->_oArticle = $article;
    }

}
