<?php

/**
 * mo_viaebay__oxorder
 * 
 * @version   $Id: mo_viaebay__oxorder.php 213 2013-03-15 11:29:15Z $
 */

/**
 * add via ebay functionality to oxorder
 * 
 * @package   mo_viaebay
 * @extend oxorder
 */
class mo_viaebay__oxorder extends mo_viaebay__oxorder_parent
{
    protected $VIAMappingArray = array('DE_Paket' => 'Paketversand',
        'DE_DeutschePostBrief' => 'Deutsche Post Brief',
        "DE_DHLPackchen" => "DHL Päckchen",
        "DE_Express" => "Expressversand",
        "DE_DHLPaket" => "DHL Paket",
        "DE_DPBuecherWarensendung" => "Deutsche Post Bücher-/Warensendung",
        "DE_HermesPaket" => "Hermes Paket",
        "DE_IloxxTransport" => "iloxx Transport",
        "DE_SonstigeDomestic" => "Sonstige 1-2 Werktage",
        "DE_Einschreiben" => "Einschreiben (inkl. aller Gebühren)",
        "DE_Nachname" => "Nachnahme (inkl. aller Gebühren)",
        "DE_SpecialDelivery" => "Sonderversand",
        "DE_SparversandAusDemAusland" => "Sparversand aus dem Ausland",
        "DE_StandardversandAusDemAusland" => "Standardversand aus dem Ausland",
        "DE_ExpressversandAusDemAusland" => "Expressversand aus dem Ausland",
        "DE_DPDExpress" => "DPD Express",
        "DE_UPSExpress" => "UPS Express",
        "DE_GLSPaket" => "GLS Paket",
        "DE_EbayHermesPaketShop2Shop" => "eBay Hermes Paket Shop2Shop",
        "DE_eBayHermesPaketSperrgutShop2Shop" => "eBay Hermes Paket Sperrgut Shop2Shop (Abgabe und Zustellung im Paketshop)",
        "DE_HermesPaketSperrgut" => "Hermes Paket Sperrgut",
        "DE_DPDClassic" => "DPD Classic",
        "DE_UPSStandard" => "UPS Standard",
        "DE_DeutschePostBriefInternational" => "Deutsche Post Brief International",
        "DE_DHLPackchenInternational" => "DHL Päckchen International",
        "DE_PaketInternational" => "Standard International Versand",
        "DE_ExpressInternational" => "Express International Versand",
        "DE_DHLPaketInternational" => "DHL Paket International",
        "DE_IloxxTransportInternational" => "iloxx Transport International",
        "DE_SonstigeInternational" => "Economy International Versand",
        "DE_HermesPaketInternational" => "Hermes Paket International",
        "DE_DeutschePostBriefIntlEcon" => "Deutsche Post Presse und Bücher Economy",
        "DE_DeutschePostBriefIntlPri" => "Deutsche Post Presse und Bücher Priority",
        "PromotionalShippingMethod" => "Versand zum Sonderpreis");
    /**
     * prevent parent call when called in via ebay script context
     * 
     * @extend  _setPayment
     * @param string $sPaymentid
     * @return boolean 
     */
    protected function _setPayment($sPaymentid)
    {
        if (!defined('MO_VIAEBAY__IS_SCRIPT_CONTEXT')) {
            return parent::_setPayment($sPaymentid);
        }
        return true;
    }

    /**
     * prevent parent call when called in via ebay script context
     * 
     * @extend _executePayment
     * @param oxBasket $oBasket
     * @param oxPayment $oUserpayment
     * @return boolean 
     */
    protected function _executePayment(oxBasket $oBasket, $oUserpayment)
    {
        if (!defined('MO_VIAEBAY__IS_SCRIPT_CONTEXT')) {
            return parent::_executePayment($oBasket, $oUserpayment);
        }
        return true;
    }

    /**
     * prevent parent call when called in via ebay script context
     * 
     * @extend _sendOrderByEmail
     * @param oxUser $oUser
     * @param oxBasket $oBasket
     * @param oxPayment $oPayment
     * @return boolean 
     */
    protected function _sendOrderByEmail($oUser = null, $oBasket = null, $oPayment = null)
    {
        if (!defined('MO_VIAEBAY__IS_SCRIPT_CONTEXT')) {
            return parent::_sendOrderByEmail($oUser, $oBasket, $oPayment);
        }
        return true;
    }

    /**
     * prevent parent call when called in via ebay script context
     * 
     * @extend validatePayment
     * @param oxBasket $oBasket
     * @return null 
     */
    public function validatePayment($oBasket)
    {
        if (!defined('MO_VIAEBAY__IS_SCRIPT_CONTEXT')) {
            return parent::validatePayment($oBasket);
        }
        return null;
    }

    /**
     * prevent parent call when called in via ebay script context
     * 
     * @extend validateDeliveryAddress
     * @param oxUser $oUser
     * @return mixed 
     */
    public function validateDeliveryAddress($oUser)
    {
        if (!defined('MO_VIAEBAY__IS_SCRIPT_CONTEXT')) {
            return parent::validateDeliveryAddress($oUser);
        }
        return null;
    }

    /**
     * check if order is via ebay order
     *
     * @return boolean 
     */
    public function mo_viaebay__isViaEbayOrder()
    {
        return (bool) $this->oxorder__mo_viaebay__via_id->value;
    }

    /**
     * load order by via id
     *
     * @param string $id
     * @return mixed 
     */
    public function mo_viaebay__loadByViaId($id)
    {
        $oxDb = oxDb::getDb();

        $sql = "SELECT " . $this->getSelectFields() . "
            FROM " . $this->getViewName() . "
            WHERE MO_VIAEBAY__VIA_ID = " . $oxDb->quote($id);

        return $this->_isLoaded = $this->assignRecord($sql);
    }

    /**
     * return required fiel from table: mo_viaebay__order_details
     *
     * @param string $field
     * @return mixed 
     */
    protected function mo_viaebay__getFieldFromDetailTable($field)
    {
        $oxDb = oxDb::getDb();

        $sql = "SELECT " . $field . "
            FROM mo_viaebay__order_details
            WHERE 
            OXID = " . $oxDb->quote($this->getId());
        return $oxDb->getOne($sql);
    }

    /**
     * save specified field to table: mo_viaebay__order_details
     *
     * @param string $field
     * @param mixed $data 
     */
    protected function mo_viaebay__saveFieldToDetailTable($field, $data)
    {
        $oxDb = oxDb::getDb();
        $sql = "INSERT INTO
            mo_viaebay__order_details 
            SET 
            OXID = " . $oxDb->quote($this->getId()) . ",
            MO_VIAEBAY__VIA_ID = " . $oxDb->quote($this->oxorder__mo_viaebay__via_id->value) . ",
            " . $field . " = " . $oxDb->quote($data) . "
            ON DUPLICATE KEY UPDATE
            " . $field . " = " . $oxDb->quote($data);

        $oxDb->execute($sql);
    }

    /**
     * get via ebay order details
     *
     * @return array 
     */
    public function mo_viaebay__getOrderDetails()
    {
        if (!$result = $this->mo_viaebay__getFieldFromDetailTable('MO_VIAEBAY__ORDER_DETAILS')) {
            return array();
        }
        return unserialize($result);
    }

    /**
     * save via ebay order details
     *
     * @param array $details 
     */
    public function mo_viaebay__saveOrderDetails($details)
    {
        $this->mo_viaebay__saveFieldToDetailTable('MO_VIAEBAY__ORDER_DETAILS', serialize($details));
    }

    /**
     * get via ebay order history
     *
     * @return string 
     */
    public function mo_viaebay__getOrderHistory()
    {
        if (!$result = $this->mo_viaebay__getFieldFromDetailTable('MO_VIAEBAY__HISTORY')) {
            return '';
        }
        return $result;
    }

    /**
     * add entry to via ebay order history
     *
     * @param string $message 
     */
    public function mo_viaebay__addToHistory($message)
    {
        $history = $this->mo_viaebay__getOrderHistory();
        $history .= date('Y-m-d H:i:s', time()) . ': ' . $message . "\n" . str_repeat('-', 25) . "\n";

        $this->mo_viaebay__saveFieldToDetailTable('MO_VIAEBAY__HISTORY', $history);
    }

    /**
     * report payment to via
     *
     * @return mixed 
     */
    public function mo_viaebay__reportPayment()
    {
//do not do anything if flag is already set
        if ($this->oxorder__mo_viaebay__payment_state->value) {
            return;
        }

        try {
            $this->oxorder__mo_viaebay__payment_state = new oxField(1);

            if ($this->oxorder__oxpaid->value == "0000-00-00 00:00:00") {
                $this->blIsPaid = true;
                $this->oxorder__oxpaid = new oxField(oxRegistry::get("oxUtilsDate")->formatDBDate(date('Y-m-d H:i:s')));
            }

            mo_viaebay__main::getInstance()->getInterface()->reportPayment($this);
            $this->mo_viaebay__addToHistory('Payment state set');
            $this->save();
        } catch (oxException $e) {
            $this->mo_viaebay__addToHistory('Error during setting of payment state');
        }
    }

    /**
     * report shipment to via
     *
     * @return mixed 
     */
    public function mo_viaebay__reportShipment()
    {
        try { 
            $this->oxorder__mo_viaebay__shipping_state = new oxField(1);
            mo_viaebay__main::getInstance()->getInterface()->reportShipment($this);
            $this->mo_viaebay__addToHistory('Shipping state set');
            $oxOrder = oxNew('oxorder');
            $oxOrder->load($this->oxorder__oxid->value);
            $viaOrderDetails = $oxOrder->mo_viaebay__getOrderDetails();
            $shippingService = $viaOrderDetails['ShippingService']; 
            $trackCode = $this->oxorder__oxtrackcode->value;
            if ((!empty($shippingService)) && (!empty($trackCode))) {
                $articlearray = $this->getOrderArticles()->getArray();
                foreach ($articlearray as $value) {
                    mo_viaebay__main::getInstance()->getInterface()->reportTrackingcode($value, $trackCode);
                    mo_viaebay__main::getInstance()->getInterface()->reportCarrierUsed($value, $shippingService);
                }
                $this->oxorder__mo_viaebay__tracking_code = new oxField($trackCode);
                $this->mo_viaebay__addToHistory('Trackingcode and CarrierUsed set');
            }
            $this->save();
        } catch (oxException $e) {
            $this->mo_viaebay__addToHistory('Error during setting of shipping state');
        }
    }
    
    public function mapVIAShippingService($shippingService){
        return $this->VIAMappingArray[$shippingService];
    }

    /**
     * set oxid order date to via sales date
     *
     * @param string $viaOrderDate 
     */
    public function mo_viaebay__setOrderDate($viaOrderDate)
    {
        $oDb = oxDb::getDb();
        $sQ = 'update oxorder set oxorderdate=\'' . $viaOrderDate . '\' where oxid=' . $oDb->quote($this->getId());
        $this->oxorder__oxorderdate = new oxField($sDate, oxField::T_RAW);
        $oDb->execute($sQ);
    }

    /**
     * prevent parent call when called in via ebay script context
     * 
     * @extend validateDeliveryAddress
     * @param oxUser $oUser
     * @return mixed 
     */
    public function validateDelivery($oBasket)
    {
        if (!defined('MO_VIAEBAY__IS_SCRIPT_CONTEXT')) {
            return parent::validateDelivery($oBasket);
        }
        return null;
    }

    /**
     * prevent parent call when called in via ebay script context
     * 
     * @extend validateDeliveryAddress
     * @param oxUser $oUser
     * @return mixed 
     */
    public function validateBasket($oBasket)
    {
        if (!defined('MO_VIAEBAY__IS_SCRIPT_CONTEXT')) {
            return parent::validateBasket($oBasket);
        }
        return null;
    }
    
        /**
     * set oxid order payment to via sales payment
     *
     * @param string $paymentName 
     */
    public function mo_viaebay__setPayment($paymentName)
    {
        $oDb = oxDb::getDb();
        $sQ = 'update oxorder set oxpaymenttype=\''. $paymentName .'\'where oxid=' . $oDb->quote($this->getId());
        $oDb->execute($sQ);
    }
           /**
     * set oxid order payment to via sales payment
     *
     * @param string $shippingService 
     */
    public function mo_viaebay__setShipping($shippingService)
    {
        $oDb = oxDb::getDb();
        $sQ = 'update oxorder set oxdeltype=\''. $shippingService .'\'where oxid=' . $oDb->quote($this->getId());
        $oDb->execute($sQ);
    }
    
    /**
     * store via data
     */
    public function save()
    {
        $result = parent::save();
        
        if (!$result) {
           return $result; 
        }
        
        if (!isAdmin()) {
            return $result;
        }
        
        if (!isset($this->oxorder__mo_viaebay__tracking_code)) {
            return $result;
        }
        
        $oxDb = oxDb::getDb();
        $sql = "UPDATE mo_viaebay__order_details
                SET mo_viaebay__tracking_code = " . $oxDb->quote($this->oxorder__mo_viaebay__tracking_code->value) . "
                WHERE
                OXID = " . $oxDb->quote($this->getId());
        $oxDb->execute($sql);
        
        return $result;
    }
    
    /**
     * create "empty" oxuserpayment object for paypal-module
     * otherwise error in oepaypaloxorder::_setOrderStatus()
     */
    public function getPaymentType()
    {
        if (!defined('MO_VIAEBAY__IS_SCRIPT_CONTEXT')) {
            return parent::getPaymentType();
        }
        
        if (!parent::getPaymentType()) {
            return oxNew('oxuserpayment');
        }
    }
}
