<?php

/**
 * mo_viaebay__pull_orders
 * 
 * script that import via orders
 * 
 * @package   mo_viaebay
 * @version   $Id: $ 
 */
include_once dirname(__FILE__) . '/bootstrap.php';

$feed = mo_viaebay__main::getInstance()->getFeed('orders');

$logger = mo_viaebay__main::getInstance()->getLogger();
$logger->info('executing task sync_orders');
$syncManager = mo_viaebay__main::getInstance()->getSyncManager('orders');

foreach ($feed->getData() as $viaOrder) {
    try {
        $logger->info('Processing VIA-order ' . $viaOrder['Id']);
        $syncManager->import($viaOrder);
    } catch (oxException $oEx) {
        $logger->error($oEx->getMessage());
    }
}

//sync tracking codes
$sql = "SELECT a.OXID, a.OXTRACKCODE, b.MO_VIAEBAY__ORDER_DETAILS, b.MO_VIAEBAY__VIA_ID, b.MO_VIAEBAY__TRACKING_CODE
        FROM " . getViewName('oxorder') . " a
        INNER JOIN mo_viaebay__order_details b
        ON a.OXID = b.OXID
        WHERE 
        a.OXSENDDATE != '0000-00-00 00:00:00' AND
        b.MO_VIAEBAY__TRACKING_CODE != a.OXTRACKCODE AND
        a.MO_VIAEBAY__VIA_ID IS NOT NULL";
$result = oxDb::getDb(oxdb::FETCH_MODE_ASSOC)->getAssoc($sql);
foreach ($result as $key => $orderData) {
    try {
        $oxOrder = oxNew('oxorder');
        $oxOrder->load($key);
        $viaOrderDetails = $oxOrder->mo_viaebay__getOrderDetails();
        if ($shippingService = $viaOrderDetails['ShippingService']) {
            $logger->info('Updating TrackingCode for viaID: ' . $oxOrder->oxorder__mo_viaebay__via_id->value);
            foreach ($oxOrder->getOrderArticles() as $article) {
                mo_viaebay__main::getInstance()->getInterface()->reportTrackingcode($article, $orderData['OXTRACKCODE']);
                mo_viaebay__main::getInstance()->getInterface()->reportCarrierUsed($article, $shippingService);
            }
            $oxOrder->oxorder__mo_viaebay__tracking_code = new oxField($orderData['OXTRACKCODE']);
            //report Trackingcode and CarrierUsed will set shipping state to 1 in via
            $oxOrder->oxorder__mo_viaebay__shipping_state = new oxField(1);
            $oxOrder->save();
        }
    } catch (Exception $e) {
        $logger->error($e->getMessage());
    }
}

//sync shipping state, if TrackingCode hasnt changed, but SendDate is set and Shipping State was not send to via.
$sql = "SELECT a.OXID, a.OXSENDDATE
        FROM " . getViewName('oxorder') . " a
        WHERE 
        a.OXSENDDATE != '0000-00-00 00:00:00' AND
        a.MO_VIAEBAY__SHIPPING_STATE IS NULL AND 
        a.MO_VIAEBAY__VIA_ID IS NOT NULL";
$sendorders = oxDb::getDb(oxdb::FETCH_MODE_ASSOC)->getAssoc($sql);
foreach ($sendorders as $key => $orderData) {
    try {
            $oxOrder = oxNew('oxorder');
            $oxOrder->load($key);
            $logger->info('Send ShippingState for viaID: ' . $oxOrder->oxorder__mo_viaebay__via_id->value);
            $oxOrder->oxorder__mo_viaebay__shipping_state = new oxField(1);
            mo_viaebay__main::getInstance()->getInterface()->reportShipment($oxOrder);
            $oxOrder->save();
        }
        catch (Exception $e) {
        $logger->error($e->getMessage());
    }
}

// sync Paid date
$sql = "SELECT OXID, OXPAID
        FROM " . getViewName('oxorder') . " 
        WHERE 
        OXPAID != '0000-00-00 00:00:00' AND
        MO_VIAEBAY__PAYMENT_STATE IS NULL AND
        MO_VIAEBAY__VIA_ID IS NOT NULL";
if ($orders = oxDb::getDb(oxdb::FETCH_MODE_ASSOC)->getAssoc($sql)) {
    foreach ($orders as $key => $orderData) {
        try {
            $oxOrder = oxNew('oxOrder');
            $oxOrder->load($key);
            $logger->info('Sending payment state for viaID: ' . $oxOrder->oxorder__mo_viaebay__via_id->value);
            $oxOrder->mo_viaebay__reportPayment();
            $oxOrder->oxorder__mo_viaebay__payment_state = new oxField(1);
            $oxOrder->save();
        } catch (Exception $ex) {
            
        }
    }
}


mo_viaebay__main::getInstance()->getTaskManager()->markTaskAsFinished('pull_orders');

$logger->info('finished task sync_orders');

