<?php

/**
 * mo_viaebay__sync_articles
 * 
 * script that syncs articles with via api
 * 
 * @package   mo_viaebay
 * @version   $Id: mo_viaebay__sync_articles.php 212 2013-03-15 11:19:03Z $ 
 */
define('MO_VIAEBAY__OUTPUT_LOG', false);
include_once dirname(__FILE__) . '/bootstrap.php';

$logger = mo_viaebay__main::getInstance()->getLogger();
$logger->info('Sync-Manager: executing task sync_articles');
$report = array(
    'success' => array(),
    'failed' => array(),
);

function getSyncArticles($offset, $limit)
{
    if($offset >= $limit) {
        return null;
    }
    //get local list - no need for offset in SQL because of MO_VIAEBAY__LAST_SYNC condition which is set after each sync
    $oxArticleList = oxNew('oxarticlelist');
    $oxArticleList->mo_viaebay__getList();
    if (!count($oxArticleList)) {
        return null;
    }
    return $oxArticleList;
}

try {
    $viaMain = mo_viaebay__main::getInstance();
    $viaConfig = $viaMain->getViaebayConfig();
    $i = 0;
    
    while($oxArticleList = getSyncArticles($i * mo_viaebay__main::MO_VIAEBAY__TICK_SIZE, $viaConfig['tasks']['sync_articles']['syncNumber'])) {
        $i++;
        $ids = array_keys($oxArticleList->getArray());
        $syncManager = mo_viaebay__main::getInstance()->getSyncManager('articles', $ids);

        foreach ($oxArticleList as $article) {
            $msg = $article->oxarticles__oxtitle->value . ' (' . $article->oxarticles__oxartnum->value . ')';
            if ($syncManager->syncObject($article)) {
                $report['success'][] = $msg;
            } else {
                $report['failed'][] = $msg;
            }
            $article->mo_viaebay__flush();
        }
        unset($oxArticleList);
    }
    
    if (empty($viaConfig['skip_article_deletion'])) {
        $logger->info('Sync-Manager: delete articles');
        $allOxidArticleViaIds = oxNew('oxarticlelist');
        $allOxidArticleViaIds = $allOxidArticleViaIds->mo_viaebay__getAllViaEbayIdsFromActiveArticles();
        $viaArticleIdsFeed = mo_viaebay__main::getInstance()->getFeed('article_ids');
        $syncManager = mo_viaebay__main::getInstance()->getSyncManager('articles');
        $syncManager->deleteAsyncData($allOxidArticleViaIds, $viaArticleIdsFeed);
    }
    
    mo_viaebay__main::getInstance()->getTaskManager()->markTaskAsFinished('sync_articles');
    
} catch (oxException $oEx) {
    $logger->error($oEx->getMessage());
}

$logger->info('Sync-Manager: finished task sync_articles'); 

foreach ($report['success'] as $success) {
    echo '<pre>OK: ' . $success . '</pre>';
}

foreach ($report['failed'] as $failed) {
    echo '<pre>Failed: ' . $failed . '</pre>';
}
echo '<pre>----------------------------------------------------</pre>';
echo '<pre>Erfolgreich: ' . count($report['success']) . '</pre>';
echo '<pre>Fehlgeschlagen: ' . count($report['failed']) . '</pre>';
